import logging
import os
from datetime import datetime
from os import path
from typing import Tuple
import PySimpleGUI as sg
import requests

from src.api import oking
import src.api.okinghub as api_okinghub
import sys

from src.entities.log import Log
from src.interface_grafica import exibir_janela_shortname
from src.layout import layout_shortname, layout_token

global is_connected_oracle_client, client_data, start_time, shortname_interface, token_interface, conexao, token_param, \
    nome_token, createDatabase
createDatabase = False
shortname_interface = ''
token_interface = ''
nome_token = ''
client_not_exists = True
exibir_interface_grafica = True
token_param = ''
conexao = False
print_payloads: bool = False
logging.basicConfig(level=logging.DEBUG,
                    format='[%(levelname)s][%(asctime)s] %(message)s',
                    datefmt='%Y-%m-%d %I:%M:%S')
logger = logging.getLogger()

def print_version():
    f = open(path.abspath('version.txt'), 'r')
    v = f.read()
    print(v)
    f.close()
    exit(0)


def get_token_from_params(args: list) -> Tuple[str, bool]:
    global usrMaster, pwdMaster, exibir_interface_grafica

    if len(args) >= 2:
        if args.__contains__('-p') or args.__contains__('--payload'):
            global print_payloads
            print_payloads = True

        #Modo de Console
        if args.__contains__('--console'):
            exibir_interface_grafica = False
            if(len(args)==3):
                if args[2].__contains__('-t='):
                    global token_param
                    token_param = args[2].replace("-t=", "")

        #Modo de Criação de Banco de Dados
        if args.__contains__('--database'):
            exibir_interface_grafica = False
            src.createDatabase = True
            # Se [database] então precisa da Usuário e Senha [MASTER]
            if len(args) < 5:
                logger.error('======================= MODO DATABASE ========================')
                logger.error('=== Necessita dos parâmetros de Usuário e Senha master     ===')
                logger.error(' uso: oking [token] [--database] [usuarioMaster] [senhaMaster]')
                logger.error(' ')
                logger.error('--------------------------------------------------------------')
                exit(1)
            #Seta Usuário e Senha Master
            usrMaster = args[3]
            pwdMaster = args[4]


        if args[1] == '--version':
            print_version()

        if len(args) >= 3 and args[2] == '--dev':
            return args[1], True

        return '', False

    elif len(args) >= 1:
        return '', False

    else:
        logger.error('Informe o token da integracao como parametro')
        exit(1)


token_oking, is_dev = get_token_from_params(sys.argv)

start_time = datetime.now().isoformat()
logger.info('Iniciando oking __init__')
logger.info(f'Ambiente: {"Dev" if is_dev else "Prod"}')

try:

    a = False
    if exibir_interface_grafica:
        a = exibir_janela_shortname()

    else:
        if os.path.isfile('shortname.txt'):
            shortname_file = open("shortname.txt", "r")
            shortname_interface = shortname_file.read()
            shortname_file.close()

        else:
            while True:
                shortname_interface = input('Digite o Shortname: ')
                try:
                    requests.get(f'https://{shortname_interface}.oking.openk.com.br')
                    with open('shortname.txt', 'w') as f:
                        f.write(shortname_interface)
                        f.close()
                        break
                except:
                    print('Shortname inválido!')
                    continue

        if token_param:
            token_interface = token_param

        elif os.path.isfile('token.txt'):
            token_file = open("token.txt", "r")
            nome_token = token_file.readline()
            value = 0
            if nome_token.__contains__('#'):
                value = nome_token.index('#') + 1
            token_interface = nome_token[value:].replace("\n", "")
            token_file.close()

        else:
            nome = input('Informe o nome que deseja para essa integração (a sua escolha): ')
            while True:
                token_interface = input('Informe o Token: ')
                nome_token = nome + '#' + token_interface + "\n"
                try:
                    client_data = oking.get(f'https://{shortname_interface}.oking.openk.com.br/api/consulta/oking_hub'
                                            f'/filtros?token={token_interface}',None)
                    client_data['url_oking'] = f'https://{shortname_interface}.oking.openk.com.br'
                    client_not_exists = False
                    with open('token.txt', 'w') as f:
                        f.write(nome_token)
                        f.close()
                        break
                except:
                    print('Token inválido!')
                    continue


    # if shortname_interface is None or token_interface is None:
    #     exit()

    if not is_dev:
        # Consultar dados da integracao do cliente (modulos, tempo de execucao, dados api okvendas)
        if client_not_exists:
            client_data = oking.get(f'https://{shortname_interface}.oking.openk.com.br/api/consulta/oking_hub'
                                    f'/filtros?token={token_interface}', None)
            client_data['url_oking'] = f'https://{shortname_interface}.oking.openk.com.br'
        if (createDatabase):
            client_data['user'] = usrMaster
            client_data['password'] = pwdMaster
        api_okinghub.post_log(Log(f'Oking inicializando {client_data["integracao_nome"]}', 'INICIALIZACAO',
                                  'INICIALIZACAO', f'{client_data.get("integracao_id")}', 'X'
                                  , F'{client_data.get("seller_id")}'))
    else:
        # Consultar dados da integracao do cliente em ambiente node-red local
        client_data = oking.get(f'http://127.0.0.1:1880/api/consulta/integracao_oking/filtros', None)
        api_okinghub.post_log(Log(f'Oking inicializando cliente id {client_data["integracao_nome"]}', '',
                                  'INICIALIZACAO', {client_data["integracao_id"]}, 'X'
                                  , {client_data["seller_id"]}))


    if client_data is not None:
        assert client_data['integracao_id'] is not None, 'Id da integracao nao informado (Api Oking)'
        assert client_data['db_type'] is not None, 'Tipo do banco de dados nao informado (Api Oking)'
        assert client_data['host'] is not None, 'Host do banco de dados nao informado (Api Oking)'
        assert client_data['database'] is not None, 'Nome do banco de dados nao informado (Api Oking)'
        assert client_data['user'] is not None, 'Usuario do banco de dados nao informado (Api Oking)'
        assert client_data['password'] is not None, 'Senha do banco de dados nao informado (Api Oking)'
        if client_data['operacao'].lower().__contains__('mplace'):
            assert client_data['url_api_principal'] is not None, 'Url Principal da api okvendas nao informado (Api Oking)'
            assert client_data['url_api_secundaria'] is not None, 'Url Secundária da api okvendas nao informado (Api Oking)'
            assert client_data['token_api_integracao'] is not None, 'Token da api Parceiro nao informado '
        assert client_data['token_oking'] is not None, 'Token da Api Oking nao informado '

        is_connected_oracle_client = False
    else:
        logger.warning(f'Cliente nao configurado no painel oking para o token: {token_interface}')

except Exception as e:
    logger.error(f'Erro: {str(e)}')
    if(a):
        exit()
    else:
        sg.popup_error(f'Problema ao executar:{str(e)} ')
        exit()
