import threading
import src
from time import sleep
import schedule
import logging
from src import interface_grafica
from src.jobs import product_jobs, order_jobs, deliver_jobs, photo_jobs
from src.jobs import system_jobs
from src.jobs import stock_jobs
from src.jobs import price_jobs
from src.jobs import sent_jobs
from src.api import oking
from src.jobs.config_jobs import enviar_criacao
from src.jobs.system_jobs import OnlineLogger
from src.jobs import client_jobs

logger = logging.getLogger()
send_log = OnlineLogger.send_log
lock = threading.Lock


# region CLIENTES

def instantiate_send_cliente_job(job_config: dict) -> None:
    """
    Instancia o job de envio dos produtos para a OKING HUB
    Args:
        job_config: Configuração do job obtida na api do oking
    """
    logger.info(job_config.get('job_name') + ' | Iniciando execucao')
    client_jobs.job_send_clients(job_config)


def run_thread_instantiate_send_cliente_job(job_config: dict) -> None:
    """
    Instancia o job de envio dos produtos para a OKING HUB
    Args:
        job_config: Configuração do job obtida na api do oking
    """
    logger.info(f'Adicionando job {job_config.get("job_name")} na thread')
    job_thread = threading.Thread(target=client_jobs.job_send_clients, args=[job_config])
    job_thread.start()


# endregion CLIENTES

# region PRODUTOS
def instantiate_send_produto_job(job_config: dict) -> None:
    """
    Instancia o job de envio dos produtos para a OKING HUB
    Args:
        job_config: Configuração do job obtida na api do oking
    """
    logger.info(job_config.get('job_name') + ' | Iniciando execucao')
    product_jobs.job_send_products(job_config)


def run_thread_instantiate_send_produto_job(job_config: dict) -> None:
    """
    Instancia o job de envio dos produtos para a OKING HUB
    Args:
        job_config: Configuração do job obtida na api do oking
    """
    logger.info(f'Adicionando job {job_config.get("job_name")} na thread')
    job_thread = threading.Thread(target=product_jobs.job_send_products, args=[job_config])
    job_thread.start()


# endregion PRODUTOS

# region Estoques
# def instantiate_insert_stock_semaphore_job(job_config: dict) -> None:
#     """
#     Instancia o job de inserção/atualização de estoques na tabela de semáforo
#     Args:
#         job_config: Configuração do job obtida na api do oking
#     """
#     logger.info(job_config.get('job_name') + ' | Iniciando execucao')
#     stock_jobs.job_insert_stock_semaphore(job_config)
#
#
# def run_thread_instantiate_insert_stock_semaphore_job(job_config: dict) -> None:
#     """
#     Instancia o job de inserção/atualização de estoques na tabela de semáforo
#     Args:
#         job_config: Configuração do job obtida na api do oking
#     """
#     logger.info(f'Adicionando job {job_config.get("job_name")} na thread')
#     job_thread = threading.Thread(target=stock_jobs.job_insert_stock_semaphore, args=[job_config])
#     job_thread.start()


def instantiate_send_stock_job(job_config: dict) -> None:
    """
    Instancia o job de envio dos estoques para a api okVendas
    Args:
        job_config: Configuração do job obtida na api do oking
    """
    logger.info(job_config.get('job_name') + ' | Iniciando execucao')
    stock_jobs.job_send_stocks(job_config)


def run_thread_instantiate_send_stock_job(job_config: dict) -> None:
    """
    Instancia o job de envio dos estoques para a api okVendas
    Args:
        job_config: Configuração do job obtida na api do oking
    """
    logger.info(f'Adicionando job {job_config.get("job_name")} na thread')
    job_thread = threading.Thread(target=stock_jobs.job_send_stocks, args=[job_config])
    job_thread.start()


# endregion Estoques

# region Precos
# def instantiate_insert_price_semaphore_job(job_config: dict) -> None:
#     """
#     Instancia o job de envio dos preços para o banco semáforo
#
#     Args:
#         job_config: Configuração do job obtida na api do oking
#     """
#     logger.info(job_config.get('job_name') + ' | Iniciando execucao')
#     price_jobs.job_insert_prices_semaphore(job_config)
#
# def run_thread_instantiate_insert_price_semaphore_job(job_config: dict) -> None:
#     """
#     Instancia o job de envio dos preços para o banco semáforo
#
#     Args:
#         job_config: Configuração do job obtida na api do oking
#     """
#     logger.info(f'Adicionando job {job_config.get("job_name")} na thread')
#     job_thread = threading.Thread(target=price_jobs.job_insert_prices_semaphore, args=[job_config])
#     job_thread.start()


def instantiate_send_price_job(job_config: dict) -> None:
    """
    Instancia o job de envio dos preços para a api okVendas

    Args:
        job_config: Configuração do job obtida na api do oking
    """
    logger.info(job_config.get('job_name') + ' | Iniciando execucao')
    price_jobs.job_send_prices(job_config)


def run_thread_instantiate_send_price_job(job_config: dict) -> None:
    """
    Instancia o job de envio dos preços para a api okVendas

    Args:
        job_config: Configuração do job obtida na api do oking
    """
    logger.info(f'Adicionando job {job_config.get("job_name")} na thread')
    job_thread = threading.Thread(target=price_jobs.job_send_prices, args=[job_config])
    job_thread.start()


# endregion Precos

# region PEDIDOS

def instantiate_orders_job(job_config: dict) -> None:
    """
    Instancia o job de envio dos pedidos para o ERP, incluindo pedidos ainda não pagos
    Args:
        job_config: Configuração do job obtida na api do oking
    """
    logger.info(job_config.get('job_name') + ' | Iniciando execucao')
    order_jobs.define_job_start(job_config)


def run_thread_instantiate_orders_job(job_config: dict) -> None:
    """
    Instancia o job de envio dos pedidos para o ERP, incluindo pedidos ainda não pagos
    Args:
        job_config: Configuração do job obtida na api do oking
    """
    logger.info(f'Adicionando job {job_config.get("job_name")} na thread')
    job_thread = threading.Thread(target=order_jobs.define_job_start, args=[job_config])
    job_thread.start()


def instantiate_paid_orders_job(job_config: dict) -> None:
    """
    Instancia o job de envio dos pedidos pagos para o ERP
    Args:
        job_config: Configuração do job obtida na api do oking
    """
    logger.info(job_config.get('job_name') + ' | Iniciando execucao')
    order_jobs.define_job_start(job_config)


def run_thread_instantiate_paid_orders_job(job_config: dict) -> None:
    """
    Instancia o job de envio dos pedidos pagos para o ERP
    Args:
        job_config: Configuração do job obtida na api do oking
    """
    logger.info(f'Adicionando job {job_config.get("job_name")} na thread')
    job_thread = threading.Thread(target=order_jobs.define_job_start, args=[job_config])
    job_thread.start()


def instantiate_forward_to_delivery(job_config: dict) -> None:
    logger.info(job_config.get('job_name') + ' | Iniciando execucao')
    sent_jobs.job_sent(job_config)


def run_thread_instantiate_forward_to_delivery(job_config: dict) -> None:
    logger.info(f'Adicionando job {job_config.get("job_name")} na thread')
    job_thread = threading.Thread(target=sent_jobs.job_sent, args=[job_config])
    job_thread.start()


def instantiate_deliver_job(job_config: dict) -> None:
    logger.info(job_config.get('job_name') + ' | Iniciando execucao')
    deliver_jobs.job_deliver(job_config)


def run_thread_instantiate_deliver_job(job_config: dict) -> None:
    logger.info(f'Adicionando job {job_config.get("job_name")} na thread')
    job_thread = threading.Thread(target=deliver_jobs.job_deliver, args=[job_config])
    job_thread.start()


def instantiate_b2b_orders_job(job_config: dict) -> None:
    """
    Instancia o job de envio dos pedidos B2B para o ERP, incluindo pedidos ainda não pagos
    Args:
        job_config: Configuração do job obtida na api do oking
    """
    logger.info(job_config.get('job_name') + ' | Iniciando execucao')
    order_jobs.define_job_start(job_config)


def run_thread_instantiate_b2b_orders_job(job_config: dict) -> None:
    """
    Instancia o job de envio dos pedidos B2B para o ERP, incluindo pedidos ainda não pagos
    Args:
        job_config: Configuração do job obtida na api do oking
    """
    logger.info(f'Adicionando job {job_config.get("job_name")} na thread')
    job_thread = threading.Thread(target=order_jobs.define_job_start, args=[job_config])
    job_thread.start()


def instantiate_paid_b2b_orders_job(job_config: dict) -> None:
    """
    Instancia o job de envio dos pedidos pagos B2B para o ERP
    Args:
        job_config: Configuração do job obtida na api do oking
    """
    logger.info(job_config.get('job_name') + ' | Iniciando execucao')
    order_jobs.define_job_start(job_config)


def run_thread_instantiate_paid_b2b_orders_job(job_config: dict) -> None:
    """
    Instancia o job de envio dos pedidos pagos B2B para o ERP
    Args:
        job_config: Configuração do job obtida na api do oking
    """
    logger.info(f'Adicionando job {job_config.get("job_name")} na thread')
    job_thread = threading.Thread(target=order_jobs.define_job_start, args=[job_config])
    job_thread.start()


# endregion PEDIDOS


def instantiate_periodic_execution_notification(job_config: dict) -> None:
    """
    Instancia o job que realiza a notificacao de execucao da integracao para a api okvendas
    Args:
        job_config: Configuração do job
    """
    logger.info(job_config.get('job_name') + ' | Iniciando execucao')
    system_jobs.send_execution_notification(job_config)


def run_thread_instantiate_periodic_execution_notification(job_config: dict) -> None:
    """
    Instancia o job que realiza a notificacao de execucao da integracao para a api okvendas
    Args:
        job_config: Configuração do job
    """
    logger.info(f'Adicionando job {job_config.get("job_name")} na thread')
    job_thread = threading.Thread(target=system_jobs.send_execution_notification, args=[job_config])
    job_thread.start()


def instantiate_envia_notafiscal_job(job_config: dict) -> None:
    """
    Instancia o job de envio dos produtos para a OKING HUB
    Args:
        job_config: Configuração do job obtida na api do oking
    """
    logger.info(job_config.get('job_name') + ' | Iniciando execucao')
    order_jobs.job_envia_notafiscal(job_config)


def run_thread_instantiate_envia_notafiscal_job(job_config: dict) -> None:
    """
    Instancia o job de envio dos produtos para a OKING HUB
    Args:
        job_config: Configuração do job obtida na api do oking
    """
    logger.info(f'Adicionando job {job_config.get("job_name")} na thread')
    job_thread = threading.Thread(target=order_jobs.job_envia_notafiscal, args=[job_config])
    job_thread.start()


# endregion Nota Fiscal

def instantiate_entregue_job(job_config: dict) -> None:
    """
    Instancia o job de envio dos produtos para a OKING HUB
    Args:
        job_config: Configuração do job obtida na api do oking
    """
    logger.info(job_config.get('job_name') + ' | Iniciando execucao')
    # order_jobs.job_entregue_job(job_config)
    system_jobs.send_execution_notification(job_config)


def run_thread_instantiate_entregue_job(job_config: dict) -> None:
    """
    Instancia o job de envio dos produtos para a OKING HUB
    Args:
        job_config: Configuração do job obtida na api do oking
    """
    logger.info(f'Adicionando job {job_config.get("job_name")} na thread')
    job_thread = threading.Thread(target=system_jobs.send_execution_notification, args=[job_config])
    job_thread.start()


# region Nota Fiscal

# region Foto

def instantiate_send_photo_job(job_config: dict) -> None:
    """
        Instancia o job de envio das fotos para a OKING HUB
        Args:
            job_config: Configuração do job obtida na api do oking
        """
    logger.info(job_config.get('job_name') + ' | Iniciando execucao')
    photo_jobs.job_send_photo(job_config)


def run_thread_instantiate_send_photo_job(job_config: dict) -> None:
    """
        Instancia o job de envio das fotos para a OKING HUB
        Args:
            job_config: Configuração do job obtida na api do oking
        """
    logger.info(f'Adicionando job {job_config.get("job_name")} na thread')
    job_thread = threading.Thread(target=photo_jobs.job_send_photo, args=[job_config])
    job_thread.start()


# region ConfigJobs

def schedule_job(job_config: dict, time_unit: str, time: int) -> None:
    logger.info(f'Adicionando job {job_config.get("job_name")} ao schedule de {time} em {time} {time_unit}')
    if src.exibir_interface_grafica:
        func = get_job_from_name(job_config.get('job_name'))
    else:
        func = get_thread_job_from_name(job_config.get('job_name'))
    if func is not None:
        if time_unit == 'M':  # Minutos
            schedule.every(time).minutes.do(func, job_config)
        elif time_unit == 'H':  # Horas
            schedule.every(time).hours.do(func, job_config)
        elif time_unit == 'D':  # Dias
            schedule.every(time).days.do(func, job_config)


# jobs

def get_job_from_name(job_name: str):
    # Estoque
    # if job_name == 'insere_estoque_produto_semaforo_job':
    #     return instantiate_insert_stock_semaphore_job
    if job_name == 'envia_estoque_job':
        return instantiate_send_stock_job

    # PRODUTO
    if job_name == 'envia_produto_job':
        return instantiate_send_produto_job

    # Preco
    # elif job_name == 'insere_preco_produto_semaforo_job':
    #     return instantiate_insert_price_semaphore_job

    elif job_name == 'envia_preco_job':
        return instantiate_send_price_job

    # Pedido
    elif job_name == 'internaliza_pedidos_job':
        return instantiate_orders_job

    elif job_name == 'encaminhar_entrega_job':
        return instantiate_forward_to_delivery

    elif job_name == 'internaliza_pedidos_pagos_job':
        return instantiate_paid_orders_job

    elif job_name == 'internaliza_pedidos_b2b_job':
        return instantiate_b2b_orders_job

    elif job_name == 'internaliza_pedidos_pagos_b2b_job':
        return instantiate_paid_b2b_orders_job

    # CLIENTE
    elif job_name == 'envia_cliente_job':
        return instantiate_send_cliente_job

    # ENTREGA
    # elif job_name == 'entregue_job':
    #     return instantiate_deliver_job

    # Notificacao
    elif job_name == 'periodic_execution_notification':
        return instantiate_periodic_execution_notification

    # Nota Fiscal
    elif job_name == 'envia_notafiscal_job':
        return instantiate_envia_notafiscal_job

    # entregue_job
    elif job_name == 'entregue_job':
        return instantiate_entregue_job

    # Foto
    elif job_name == 'envia_foto_job':
        return instantiate_send_photo_job


def get_thread_job_from_name(job_name: str):
    # Estoque
    # if job_name == 'insere_estoque_produto_semaforo_job':
    #     return run_thread_instantiate_insert_stock_semaphore_job
    if job_name == 'envia_estoque_job':
        return run_thread_instantiate_send_stock_job

    # PRODUTO
    if job_name == 'envia_produto_job':
        return run_thread_instantiate_send_produto_job

    # Preco
    # elif job_name == 'insere_preco_produto_semaforo_job':
    #     return run_thread_instantiate_insert_price_semaphore_job

    elif job_name == 'envia_preco_job':
        return run_thread_instantiate_send_price_job

    # Pedido
    elif job_name == 'internaliza_pedidos_job':
        return run_thread_instantiate_orders_job

    elif job_name == 'encaminhar_entrega_job':
        return run_thread_instantiate_forward_to_delivery

    elif job_name == 'internaliza_pedidos_pagos_job':
        return run_thread_instantiate_paid_orders_job

    elif job_name == 'internaliza_pedidos_b2b_job':
        return run_thread_instantiate_b2b_orders_job

    elif job_name == 'internaliza_pedidos_pagos_b2b_job':
        return run_thread_instantiate_paid_b2b_orders_job

    # CLIENTE
    elif job_name == 'envia_cliente_job':
        return run_thread_instantiate_send_cliente_job

    # ENTREGA
    # elif job_name == 'entregue_job':
    #     return instantiate_deliver_job

    # Notificacao
    elif job_name == 'periodic_execution_notification':
        return run_thread_instantiate_periodic_execution_notification

    # Nota Fiscal
    elif job_name == 'envia_notafiscal_job':
        return run_thread_instantiate_envia_notafiscal_job

    # entregue_job
    elif job_name == 'entregue_job':
        return run_thread_instantiate_entregue_job

    # Foto
    elif job_name == 'envia_foto_job':
        return run_thread_instantiate_send_photo_job


# endregion ConfigJobs
token = ''

modules: list = [oking.Module(**m) for m in src.client_data.get('modulos')]
assert modules is not None, 'Nao foi possivel obter os modulos da integracao. Por favor, entre em contato com o suporte.'

for module in modules:
    if module.time is None:
        module.time = 9999
    schedule_job({
        'db_host': src.client_data.get('host'),
        'db_port': src.client_data.get('port'),
        'db_user': src.client_data.get('user'),
        'db_type': src.client_data.get('db_type'),
        'db_seller': src.client_data.get('loja_id'),
        'db_name': src.client_data.get('database'),
        'db_pwd': src.client_data.get('password'),
        'db_client': src.client_data.get('diretorio_client'),
        'operacao': src.client_data.get('operacao'),
        'send_logs': module.send_logs,
        'enviar_logs': module.send_logs,
        'job_name': module.job_name,
        'sql': module.sql,
        'semaforo_sql': module.exists_sql,
        'query_final': module.query_final,
        'ultima_execucao': module.ultima_execucao,
    }, module.time_unit, module.time)

# Job para notificar execucao periodica do Oking a cada 30 min
schedule_job({
    'job_name': 'periodic_execution_notification',
    'execution_start_time': src.start_time,
    'job_qty': len(schedule.get_jobs()),
    'integration_id': src.client_data.get('integracao_id')
}, 'M', 30)


def main():
    logger.info('Iniciando oking __main__')
    logger.info('===========================================================================')
    logger.info('==== Iniciando OKING HUB __main__')
    logger.info(f'==== Exibir Interface grafica: {src.exibir_interface_grafica}')
    logger.info('===========================================================================')

    if src.exibir_interface_grafica == False and src.createDatabase == False:
        while True:
            try:
                schedule.run_pending()
                sleep(5)
            except Exception as e:
                logger.error(f'Erro não tratado capturado: {str(e)}')
                send_log('__main__', src.client_data.get('enviar_logs'), True, f'', 'error', '')

    elif src.createDatabase:
        try:
            enviar_criacao(src.client_data)
        except Exception as e:
            logger.error(f'Erro não tratado capturado: {str(e)}')
    else:
        interface_grafica.exibir_interface()


if __name__ == "__main__":
    main()
