import logging

from src.database import utils
from src.api import okinghub as api_okHUB

import src.api.entities.cliente
import src.database.connection as database
from src.database import queries
from src.database.queries import IntegrationType
from src.database.utils import DatabaseConfig
from src.jobs.system_jobs import OnlineLogger
from threading import Lock
logger = logging.getLogger()
send_log = OnlineLogger.send_log

lock = Lock()
def job_send_clients(job_config_dict: dict):
    with lock:
        """
        Job para realizar atualização de clientes
        Args:
            job_config_dict: Configuração do job
        """
        db_config = utils.get_database_config(job_config_dict)
        # Exibe mensagem monstrando que a Thread foi Iniciada
        logger.info(f'==== THREAD INICIADA -job: ' + job_config_dict.get('job_name'))

        # LOG de Inicialização do Método - Para acompanhamento de execução
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True,
                 f'Cliente - Iniciado', 'exec', 'envia_cliente_job', 'CLIENTE')

        clients = query_clients_erp(job_config_dict, db_config)
        if clients is not None and len(clients) > 0:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True,
                     f'Produtos para atualizar cliente {len(clients)}', 'info', 'CLIENTE')

            api_clients = []
            for client in clients:
                if api_clients.__len__() < 50:
                    api_clients.append(client)
                else:
                    send_log(job_config_dict.get('job_name'), False, False, f'Enviando Pacote: {api_clients.__len__()}', 'info', 'CLIENTE')
                    response = api_okHUB.post_clients(api_clients)
                    api_clients = []
                    send_log(job_config_dict.get('job_name'), False, False, f'Tratando retorno', 'info', 'CLIENTE', '')
                    validate_response_client(response, db_config, job_config_dict)

            if api_clients.__len__() > 0:
                send_log(job_config_dict.get('job_name'), False, False, f'Enviando Pacote: {api_clients.__len__()}', 'info', 'CLIENTE', '')
                response = api_okHUB.post_clients(api_clients)
                api_clients = []
                send_log(job_config_dict.get('job_name'), False, False, f'Tratando retorno', 'info', 'CLIENTE', '')
                validate_response_client(response, db_config, job_config_dict)

        else:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, 'Não existem clientes a serem inseridos no momento', 'warning', 'CLIENTE')


def query_clients_erp(job_config_dict: dict, db_config: DatabaseConfig):
    """
    Consulta os clientes para atualizar no banco de dados
    Args:
        job_config_dict: Configuração do job
        db_config: Configuração do banco de dados

    Returns:
        Lista de clientes para atualizar
    """
    db = database.Connection(db_config)
    conn = db.get_conect()
    cursor = conn.cursor()
    try:
        newsql = utils.final_query(db_config)
        conexao = database.Connection(db_config)
        conn = conexao.get_conect()

        cursor.execute(newsql.lower())
        rows = cursor.fetchall()
        columns = [col[0] for col in cursor.description]
        results = list(dict(zip(columns, row)) for row in rows)
        cursor.close()
        conn.close()

        clients = []

        if len(results) > 0:
            clients = client_dict(results)
        return clients

    except Exception as ex:
        if src.exibir_interface_grafica:
            raise
        else:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False,
                 f' Erro ao consultar clientes no banco semaforo: {str(ex)}', 'error', 'CLIENTE')


def client_dict(clients):
    lista = []
    for row in clients:
        pdict = {
            'id': str(row['id']),
            'nome': int(row['id']),
            'token': str(src.client_data.get('token_oking')),
        }
        lista.append(pdict)

    return lista


def validate_response_client(response, db_config, job_config_dict):
    if response is not None:
        conexao = database.Connection(db_config)
        conn = conexao.get_conect()
        cursor = conn.cursor()

        # Percorre todos os registros
        for item in response:
            identificador = item.identificador
            identificador2 = item.identificador2
            if item.sucesso == 1 or item.sucesso == 'true':
                msgret = 'SUCESSO'
            else:
                msgret = item.Message
                send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True,
                         f'Erro ao atualizar Cliente para o sku: {identificador}, {msgret}', 'warning',
                         'CLIENTE', identificador)
            try:
                cursor.execute(queries.get_insert_update_semaphore_command(db_config.db_type),
                               queries.get_command_parameter(db_config.db_type, [identificador, identificador2,
                                                                                 IntegrationType.CLIENTE.value,
                                                                                 msgret]))
                # atualizados.append(response['identificador'])
            except Exception as e:
                send_log(job_config_dict.get('job_name')
                         , job_config_dict.get('enviar_logs')
                         , True
                         , f'Erro ao atualizar Cliente do sku: {item.identificador}, Erro: {str(e)}'
                         , 'error'
                         , 'CLIENTE'
                         , f'{item.identificador}-{item.identificador2}')

        cursor.close()
        conn.commit()
        conn.close()
