import logging
import string
from datetime import datetime
from threading import Lock
from time import sleep
from typing import List

import PySimpleGUI as sg

import src
import src.api.api_mplace as api_Mplace
import src.api.okinghub as api_okHUB
import src.api.okvendas as api_okVendas
import src.database.connection as database
import src.database.utils as utils
from src.api.entities.pedido import Queue, Order, OrderMplace
from src.database import queries
from src.database.utils import DatabaseConfig
from src.entities.invoice import Invoice
from src.jobs.system_jobs import OnlineLogger

logger = logging.getLogger()
send_log = OnlineLogger.send_log
lock = Lock()

default_limit = 50
queue_status = {
    'pending': 'PENDENTE',
    'AGUARDANDO_PAGAMENTO': 'AGUARDANDO_PAGAMENTO',
    'paid': 'PAGO',
    'APROVADO': 'APROVADO',
    'invoiced': 'FATURADO',
    'em_processamento': 'EM_PROCESSAMENTO',
    'canceled': 'CANCELADO',
    'shipped': 'ENCAMINHADO',
    'delivered': 'ENTREGUE',
    'paid_order': 'PEDIDO_PAGO'
}


def define_job_start(job_config: dict) -> None:
    global current_job
    current_job = job_config.get('job_name')
    try:
        # Inicia o job a partir dos pedidos AgPagamento
        if current_job == 'internaliza_pedidos_job' or current_job == 'internaliza_pedidos_b2b_job':
            with lock:
                # Exibe mensagem monstrando que a Thread foi Iniciada
                logger.info(f'==== THREAD INICIADA -job: ' + job_config.get('job_name'))
                job_orders(job_config, True)
        # Inicia o job a partir do pedidos pagos
        elif current_job == 'internaliza_pedidos_pagos_job' or current_job == 'internaliza_pedidos_pagos_b2b_job':
            with lock:
                # Exibe mensagem monstrando que a Thread foi Iniciada
                logger.info(f'==== THREAD INICIADA -job: ' + job_config.get('job_name'))
                job_orders(job_config)
    except Exception as e:
        raise e


def job_orders(job_config: dict, start_at_pending: bool = False) -> None:
    try:
        db_config = utils.get_database_config(job_config)
        if not src.client_data['operacao'].lower().__contains__('mplace'):
            if start_at_pending:
                process_order_queue(job_config, queue_status.get('pending'), db_config, True)

            process_order_queue(job_config, queue_status.get('paid'), db_config, True)

            process_order_queue(job_config, queue_status.get('canceled'), db_config)
        else:
            # MPLACE
            if start_at_pending:
                process_order_queue_mplace(job_config, queue_status.get('AGUARDANDO_PAGAMENTO'), db_config, True)

            process_order_queue_mplace(job_config, queue_status.get('APROVADO'), db_config, True)

            process_order_queue_mplace(job_config, queue_status.get('canceled'), db_config)

    except Exception as e:
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), True, f'Erro ao inicializar job: {str(e)}',
                 'error', 'PEDIDO')
        raise e


def job_envia_notafiscal(job_config: dict):
    with lock:
        try:
            db_config = utils.get_database_config(job_config)
            # Exibe mensagem monstrando que a Thread foi Iniciada
            logger.info(f'==== THREAD INICIADA -job: ' + job_config.get('job_name'))

            # LOG de Inicialização do Método - Para acompanhamento de execução
            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), True,
                     f'Enviar Nota Fiscal - Iniciado', 'exec', 'envia_notafiscal_job', 'NOTAFISCAL')

            if db_config.sql is None:
                send_log(job_config.get('job_name'), job_config.get('enviar_logs'), True,
                         f'Comando sql para baixar Notas Fiscais nao encontrado', 'warning'
                         , 'envia_notafiscal_job', 'NOTAFISCAL')
            else:
                if src.client_data['operacao'].lower().__contains__('mplace'):  # Operação Mplace
                    queue = api_Mplace.get_order_queue_mplace(src.client_data, queue_status.get('em_processamento'))

                    for q_order in queue:
                        invoices = query_invoices(job_config, db_config, q_order.pedido_oking_id)
                        # Consultar no banco de dados o pedido
                        qtd = invoices.__len__()
                        # Consulta fila pedidos processamento
                        if qtd > 0:
                            for invoice in invoices:
                                try:
                                    invoice_sent = api_Mplace.post_invoices_mplace(q_order.pedido_oking_id, invoice)
                                    if invoice_sent is None:
                                        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                                                 f'NF do pedido {invoice.id} enviada com sucesso para MPLACE',
                                                 'info','envia_notafiscal_job', 'NOTAFISCAL')

                                        db = database.Connection(db_config)
                                        conn = db.get_conect()
                                        cursor = conn.cursor()
                                        cursor.execute(queries.get_update_data_sincronizacao_nf_command(db_config.db_type),
                                                       queries.get_command_parameter(db_config.db_type, [invoice.id]))
                                        cursor.close()
                                        conn.commit()
                                        conn.close()
                                        continue
                                    else:
                                        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), True,
                                                 f'Falha ao enviar NF do pedido {invoice.id} para MPLACE:{invoice_sent}'
                                                 , 'error', 'envia_notafiscal_job', 'NOTAFISCAL')
                                        if src.exibir_interface_grafica:
                                            sg.popup(invoice_sent.message)
                                except Exception as e:
                                    send_log(job_config.get('job_name'), job_config.get('enviar_logs'), True,
                                             f'Falha ao enviar NF do pedido {invoice.id}: {str(e)}', 'error'
                                             , 'envia_notafiscal_job', 'NOTAFISCAL')
                            else:
                                send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                                         f'Nenhum pedido encontrado', 'error', 'envia_notafiscal_job', 'NOTAFISCAL')
                elif src.client_data['operacao'].lower().__contains__('okvendas'):
                    invoices = query_invoices(job_config, db_config)
                    for invoice in invoices:
                        try:
                            invoice_sent = api_okVendas.post_invoices(invoice)
                            if invoice_sent is None:
                                send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                                         f'NF do pedido {invoice.id} enviada com sucesso para api OKVendas', 'info'
                                         ,'envia_notafiscal_job', 'NOTAFISCAL')
                                continue
                            else:
                                send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                                         f'Falha ao enviar NF do pedido {invoice.id} para api okvendas: '
                                         f'{invoice_sent.message}', 'error', 'envia_notafiscal_job', 'NOTAFISCAL')
                        except Exception as e:
                            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), True,
                                     f'Falha ao enviar NF do pedido {invoice.id}: {str(e)}', 'error'
                                     , 'envia_notafiscal_job', 'NOTAFISCAL')
                        raise e
                else:  # Operação OKING
                    invoices = query_invoices(job_config, db_config)
                    for invoice in invoices:
                        try:
                            invoice_sent = api_okHUB.post_faturar(invoice)
                            if invoice_sent is None:
                                send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                                         f'NF do pedido {invoice.id} enviada com sucesso para api OKING', 'info'
                                         , 'envia_notafiscal_job', 'NOTAFISCAL')
                                continue
                            else:
                                send_log(job_config.get('job_name'), job_config.get('enviar_logs'), True,
                                         f'Falha ao enviar NF do pedido {invoice.id} para api OKING: {invoice_sent}'
                                         , 'error', 'envia_notafiscal_job', 'NOTAFISCAL')
                        except Exception as e:
                            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), True,
                                     f'Falha ao enviar NF do pedido {invoice.id}: {str(e)}', 'error'
                                     , 'envia_notafiscal_job', 'NOTAFISCAL')
                            raise e

        except Exception as e:
            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), True,
                     f'Falha na execução do job: {str(e)}', 'error', 'envia_notafiscal_job', 'NOTAFISCAL')
            raise e


# region Order - OKINGHUB
def process_order_queue(job_config: dict, status: str, db_config: DatabaseConfig,
                        status_to_insert: bool = False) -> None:
    try:
        print()
        print()
        queue = []
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                 f'Consultando fila de pedidos no status {status}', 'info', 'PEDIDO')

        # LOG de Inicialização do Método - Para acompanhamento de execução
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), True,
                 f'Pedido - Iniciado[{status}]', 'exec', 'INTERNALIZA_PEDIDOS_JOB', 'PEDIDO')

        if not src.client_data['operacao'].lower().__contains__('mplace'):
            queue = api_okHUB.get_order_queue(src.client_data, status)
        else:
            queue = api_Mplace.get_order_queue_mplace(src.client_data, status)

        qty = 0
        for q_order in queue:
            try:
                sleep(0.5)
                qty = qty + 1
                print()
                send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                         f'Iniciando processamento ({qty} de {len(queue)}) Pedido OKING ID: {q_order.pedido_oking_id}',
                         'info',  'INTERNALIZA_PEDIDOS_JOB', 'PEDIDO')
                order = api_okHUB.get_order(src.client_data, q_order.pedido_oking_id)
                # Pedido integrado anteriormente
                if order.capa.numero_pedido_externo is not None and order.capa.numero_pedido_externo != '':

                    if check_order_existence(db_config, int(order.capa.pedido_oking_id)):
                        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                                 f'Pedido ja integrado com o ERP, chamando procedure de atualizacao...', 'info',
                                 'INTERNALIZA_PEDIDOS_JOB')
                        if call_update_order_procedure(job_config, db_config, int(order.capa.pedido_oking_id),
                                                       order.capa.status):
                            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                                     f'Pedido atualizado com sucesso', 'info', 'INTERNALIZA_PEDIDOS_JOB')
                            set_codigo_erp_order(job_config, db_config=db_config, order=order, queue_order=q_order,
                                                 order_erp_id=order.capa.numero_pedido_externo, client_erp_id='')
                    else:
                        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                                 f'Pedido {order.capa.pedido_oking_id} nao existe no banco semaforo porem ja foi '
                                 f'integrado previamente. Protocolando pedido...',
                                 'warning', 'INTERNALIZA_PEDIDOS_JOB', 'PEDIDO')
                        protocol_non_existent_order(job_config, q_order)

                else:  # Pedido nao integrado anteriormente

                    if check_non_integrated_order_existence(db_config,
                                                            int(order.capa.pedido_oking_id)):  # Pedido existente no banco semaforo

                        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                                 f'Pedido já existente no banco semáforo, porem nao integrado com erp. Chamando procedures',
                                 'info', 'INTERNALIZA_PEDIDOS_JOB')
                        # Chama a Procedure para Inserir o Pedido no ERP (cliente)
                        sp_success, client_erp_id, order_erp_id = call_order_procedures(job_config, db_config,
                                                                                        int(order.capa.pedido_oking_id))
                        if sp_success:
                            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                                     f'Chamadas das procedures executadas com sucesso, protocolando pedido...', 'info',
                                     'INTERNALIZA_PEDIDOS_JOB')
                            set_codigo_erp_order(job_config, db_config=db_config, order=order, queue_order=q_order,
                                                 order_erp_id=order_erp_id, client_erp_id=client_erp_id)

                    elif status_to_insert:  # Pedido nao existe no semaforo e esta em status de internalizacao (pending e paid)

                        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                                 f'Inserindo novo pedido no banco semaforo', 'info', 'INTERNALIZA_PEDIDOS_JOB')
                        inserted = insert_temp_order(job_config, order, db_config)
                        if inserted:
                            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                                     f'Pedido inserido com sucesso, chamando procedures...', 'info', 'INTERNALIZA_PEDIDOS_JOB')
                            # Chama a Procedure para Inserir o Pedido no ERP (cliente)
                            sp_success, client_erp_id, order_erp_id = call_order_procedures(job_config, db_config,
                                                                                            int(order.capa.pedido_oking_id))
                            if sp_success:
                                send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                                         f'Chamadas das procedures executadas com sucesso, protocolando pedido...',
                                         'info',
                                         'INTERNALIZA_PEDIDOS_JOB')
                                set_codigo_erp_order(job_config, db_config=db_config, order=order, queue_order=q_order,
                                                     order_erp_id=order_erp_id, client_erp_id=client_erp_id)

                    else:  # Pedido nao existe no semaforo e nao esta em status de internalizacao (pending e paid)
                        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                                 f'Pedido nao existente no banco semaforo e nao se encontra em status de internalizacao',
                                 'warning', 'INTERNALIZA_PEDIDOS_JOB')

            except Exception as e:
                send_log(job_config.get('job_name'), job_config.get('enviar_logs'), True,
                         f'Erro no processamento do pedido {q_order.pedido_oking_id}: {str(e)}'
                         , 'error', 'PEDIDO', q_order.pedido_oking_id)
    except Exception as e:
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), True,
                 f'Erro ao inicializar job de processamento de pedidos do status {status}: {str(e)}'
                 , 'error', 'INTERNALIZA_PEDIDOS_JOB', status)
        raise


# endregion
def call_update_order_procedure(job_config: dict, db_config: DatabaseConfig, pedido_oking_id: int, status: str) -> bool:
    success = True
    db = database.Connection(db_config)
    conn = db.get_conect()
    try:
        cursor = conn.cursor()
        updated: bool = False
        if db_config.is_sql_server():
            cursor.execute('exec openk_semaforo.sp_atualiza_pedido @pedido = ?', pedido_oking_id)
            (updated,) = cursor.fetchone()
        elif db_config.is_oracle():
            order_updated = cursor.var(int)
            cursor.callproc('OPENK_SEMAFORO.SP_ATUALIZA_PEDIDO',
                            [pedido_oking_id, status, order_updated])
            updated = order_updated.getvalue()
        elif db_config.is_mysql():
            order_updated = 0
            args = (pedido_oking_id, status, order_updated)
            result_args = cursor.callproc('openk_semaforo.SP_ATUALIZA_PEDIDO', args)
            if result_args[2] > 0:
                updated = True
        if updated is None or updated <= 0:
            success = False
            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                     f'Nao foi possivel atualizar o pedido informado', 'warning', 'INTERNALIZA_PEDIDOS_JOB')

        cursor.close()
        if success:
            conn.commit()
        else:
            conn.rollback()
        conn.close()
        return success
    except Exception as e:
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), True,
                 f'Erro no método de chamada da procedure de atualização do pedido {pedido_oking_id}: {str(e)}',
                 'error',
                 'INTERNALIZA_PEDIDOS_JOB', str(pedido_oking_id))
        conn.rollback()
        conn.close()


def check_order_existence(db_config: DatabaseConfig, pedido_oking_id: int) -> bool:
    db = database.Connection(db_config)
    conn = db.get_conect()
    try:
        cursor = conn.cursor()
        cursor.execute(queries.get_query_order(db_config.db_type),
                       queries.get_command_parameter(db_config.db_type, [pedido_oking_id]))
        existent_order = cursor.fetchone()[0]
        cursor.close()
        conn.close()
        return existent_order is not None and existent_order > 0
    except Exception as e:
        conn.close()
        return False


def set_codigo_erp_order(job_config: dict, db_config: DatabaseConfig, order: Order | OrderMplace, queue_order: Queue,
                         order_erp_id: str = '', client_erp_id: str = '') -> None:
    db = database.Connection(db_config)
    try:
        conn = db.get_conect()
        cursor = conn.cursor()
        if order_erp_id != '':
            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                     f'Protocolando pedido com codigo_referencia {order_erp_id}', 'info', 'INTERNALIZA_PEDIDOS_JOB')

            if src.client_data['operacao'].lower().__contains__('mplace'):
                # colocar o pedido em processamento. [EM_PROCESSAMENTO]
                api_Mplace.post_order_processing_mplace(order.order_id)
                updated_order_code = api_Mplace.put_order_erp_code_mplace(order.order_id, order_erp_id)
            else:
                updated_order_code = api_okHUB.put_order_erp_code(
                    int(order.capa.pedido_oking_id),
                    order_erp_id)

            if updated_order_code:
                send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                         f'Codigo Erp do pedido atualizado via api', 'info', 'INTERNALIZA_PEDIDOS_JOB')
            else:
                send_log(job_config.get('job_name'), job_config.get('enviar_logs'), True,
                         f'Falha ao atualizar o Codigo Erp do pedido via api', 'warning',
                         'INTERNALIZA_PEDIDOS_JOB', order.capa.pedido_oking_id)

        if client_erp_id != '':
            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                     f'Salvando codigo do erp do cliente no banco semaforo {client_erp_id}', 'info',
                     'INTERNALIZA_PEDIDOS_JOB')

            if src.client_data['operacao'].lower().__contains__('mplace'):
                cursor.execute(queries.get_client_protocol_command(db_config.db_type),
                               queries.get_command_parameter(db_config.db_type,
                                                             [client_erp_id, order.order_id]))
            else:
                cursor.execute(queries.get_client_protocol_command(db_config.db_type),
                               queries.get_command_parameter(db_config.db_type,
                                                             [client_erp_id, order.capa.pedido_oking_id]))

            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                     f'Protocolando cliente via api okvendas com codigo_referencia {client_erp_id}', 'info'
                     , 'INTERNALIZA_PEDIDOS_JOB')

            if not src.client_data['operacao'].lower().__contains__('mplace'):
                updated_client_code = api_okHUB.put_client_erp_code(
                    {
                        'cpf_cnpj': order.usuario.cpf_cnpj if order.usuario.cpf_cnpj is not None or order.usuario.cpf_cnpj != '' else order.usuario.cpf_cnpj,
                        'codigo_cliente': client_erp_id
                    })

                if updated_client_code:
                    send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                             f'Codigo Erp do cliente atualizado via api OkVendas', 'info'
                             , 'INTERNALIZA_PEDIDOS_JOB')
                else:
                    send_log(job_config.get('job_name'), job_config.get('enviar_logs'), True,
                             f'Falha ao atualizar o Codigo Erp do cliente via api OkVendas', 'warning'
                             , 'INTERNALIZA_PEDIDOS_JOB', client_erp_id)

        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                 f'Removendo pedido da fila pelo protocolo {queue_order.protocolo}', 'info'
                 , 'INTERNALIZA_PEDIDOS_JOB')

        if src.client_data['operacao'].lower().__contains__('mplace'):
            # protocoled_order = api_Mplace.put_protocol_orders_mplace(queue_order.protocolo)
            # não há necessidade de tirar ele da fila, pois o método anterior coloca em outra fila
            protocoled_order = True
        else:
            protocoled_order = api_okHUB.put_protocol_orders([queue_order.protocolo])

        if protocoled_order:
            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                     f'Pedido protocolado via api OkVendas', 'info', 'INTERNALIZA_PEDIDOS_JOB')
        else:
            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), True,
                     f'Falha ao protocolar pedido via api OkVendas', 'warning'
                     , 'INTERNALIZA_PEDIDOS_JOB','PROTOCOLO' )

        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                 f'Protocolando pedido no banco semaforo', 'info', 'INTERNALIZA_PEDIDOS_JOB')

        if src.client_data['operacao'].lower().__contains__('mplace'):
            cursor.execute(queries.get_order_protocol_command(db_config.db_type),
                           queries.get_command_parameter(db_config.db_type, [order_erp_id, order.order_id]))
        else:
            cursor.execute(queries.get_order_protocol_command(db_config.db_type),
                           queries.get_command_parameter(db_config.db_type, [order_erp_id, order.capa.pedido_oking_id]))

        cursor.close()
        conn.commit()
        conn.close()
    except Exception as e:
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), True,
                 f'Erro ao protocolar pedidos: {str(e)}', 'error'
                 , 'INTERNALIZA_PEDIDOS_JOB', order_erp_id)


def protocol_non_existent_order(job_config: dict, queue_order: Queue) -> None:
    try:
        if src.client_data['operacao'].lower().__contains__('mplace'):
            protocoled_order = api_Mplace.put_protocol_orders_mplace(queue_order.protocolo)
            if protocoled_order:
                logger.info(current_job + f' | Pedido {queue_order.pedido_oking_id} protocolado via api OkVendas')
            else:
                logger.warning(
                    current_job + f' | Falha ao protocolar pedido {queue_order.pedido_oking_id} via api OkVendas')
        else:
            protocoled_order = api_okHUB.put_protocol_orders([queue_order.protocolo])
            if protocoled_order:
                logger.info(current_job + f' | Pedido {queue_order.pedido_oking_id} protocolado via api OkVendas')
            else:
                logger.warning(
                    current_job + f' | Falha ao protocolar pedido {queue_order.pedido_oking_id} via api OkVendas')

    except Exception as ex:
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), True,
                 f'Erro ao protocolar pedido {queue_order.pedido_oking_id}: {str(ex)}', 'error'
                 , 'INTERNALIZA_PEDIDOS_JOB', queue_order.pedido_oking_id)


def insert_temp_order(job_config: dict, order: Order, db_config: DatabaseConfig) -> bool:
    step = ''
    db = database.Connection(db_config)
    conn = db.get_conect()
    try:
        step = 'conexao'
        cursor = conn.cursor()

        existent_client = None
        if order.usuario.codigo_referencia_erp is not None and order.usuario.codigo_referencia_erp != '' and order.usuario.codigo_referencia_erp != '0':  # Por padrao o erp_code vem = '0' da api okvendas
            cursor.execute(queries.get_query_client_erp(db_config.db_type),
                           queries.get_command_parameter(db_config.db_type, [order.user.erp_code]))
            existent_client = cursor.fetchone()

        if existent_client is None:
            # insere cliente

            step = 'P.1 -Insere NOVO cliente'
            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                     f'\tPedido {order.capa.pedido_oking_id}: Inserindo cliente', 'info', 'INTERNALIZA_PEDIDOS_JOB')
            cursor.execute(queries.get_insert_client_command(db_config.db_type),
                           queries.get_command_parameter(db_config.db_type, [
                               order.usuario.nome or order.usuario.razao_social,
                               order.usuario.razao_social or order.usuario.nome,
                               order.usuario.tipo_pessoa,
                               order.usuario.cpf_cnpj,
                               order.usuario.sexo,
                               order.usuario.email,
                               order.usuario.rg,
                               order.usuario.orgao,
                               order.usuario.data_nascimento_constituicao,
                               order.usuario.telefone_residencial,
                               order.usuario.telefone_celular,
                               order.usuario.inscricao_estadual,
                               order.usuario.inscricao_municipal,
                               order.usuario.codigo_referencia_erp,
                               order.usuario.codigo_representante,
                               order.usuario.endereco_cobranca.cep,
                               order.usuario.endereco_cobranca.tipo_logradouro,
                               order.usuario.endereco_cobranca.endereco,
                               order.usuario.endereco_cobranca.numero,
                               order.usuario.endereco_cobranca.complemento,
                               order.usuario.endereco_cobranca.bairro or " ",
                               order.usuario.endereco_cobranca.cidade,
                               order.usuario.endereco_cobranca.estado,
                               order.usuario.endereco_cobranca.referencia_entrega,
                               order.usuario.endereco_cobranca.codigo_ibge,
                               'IN'
                           ]))
        else:
            # update no cliente existente
            step = 'P.1 -Atualizando Cliente Existente'
            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                     f'\tPedido {order.capa.pedido_oking_id}: Atualizando cliente existente', 'info'
                     , 'INTERNALIZA_PEDIDOS_JOB')
            cursor.execute(queries.get_update_client_sql(db_config.db_type),
                           queries.get_command_parameter(db_config.db_type, [
                               order.usuario.nome or order.usuario.razao_social,
                               order.usuario.razao_social or order.usuario.nome,
                               order.usuario.tipo_pessoa,
                               order.usuario.cpf_cnpj,
                               order.usuario.sexo,
                               order.usuario.email,
                               order.usuario.rg,
                               order.usuario.orgao,
                               order.usuario.data_nascimento_constituicao,
                               order.usuario.telefone_residencial,
                               order.usuario.telefone_celular,
                               order.usuario.inscricao_estadual,
                               order.usuario.inscricao_municipal,
                               order.usuario.codigo_referencia_erp,
                               order.usuario.codigo_representante,
                               order.usuario.endereco_cobranca.cep,
                               order.usuario.endereco_cobranca.tipo_logradouro,
                               order.usuario.endereco_cobranca.endereco,
                               order.usuario.endereco_cobranca.numero,
                               order.usuario.endereco_cobranca.complemento,
                               order.usuario.endereco_cobranca.bairro or " ",
                               order.usuario.endereco_cobranca.cidade,
                               order.usuario.endereco_cobranca.estado,
                               order.usuario.endereco_cobranca.referencia_entrega,
                               order.usuario.endereco_cobranca.codigo_ibge
                           ]))

        step = 'P.2 - Antes de Executar a Querys'
        # if cursor.rowcount > 0:
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                 f'Cliente inserido/atualizado para o pedido {order.order_id}', 'info', 'INTERNALIZA_PEDIDOS_JOB')
        cursor.execute(queries.get_query_cliente_cpfcnpj(db_config.db_type),
                       queries.get_command_parameter(db_config.db_type, [
                           order.customer.document_number.translate(str.maketrans('', '', string.punctuation))]))
        step = 'P.3 - Queries executada'
        (client_id,) = cursor.fetchone()
        if client_id is None or client_id <= 0:
            cursor.close()
            raise Exception('Nao foi possivel obter o cliente inserido do banco de dados')
        # else:
        #     cursor.close()
        #     raise Exception('O cliente nao foi inserido')

        # insere pedido
        step = 'P.4 - Inserindo Pedido'
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                 f'\tPedido {order.capa.pedido_oking_id}: Inserindo cabecalho pedido', 'info'
                 , 'INTERNALIZA_PEDIDOS_JOB')
        cursor.execute(queries.get_insert_order_command(db_config.db_type),
                       queries.get_command_parameter(db_config.db_type, [
                           order.capa.pedido_oking_id,
                           order.capa.pedido_venda_id,
                           order.capa.pedido_canal,
                           str(datetime.strptime(order.capa.data_pedido.replace('T', ' '),
                                                 '%Y-%m-%d %H:%M:%S') if order.capa.data_pedido is not None else ''),
                           order.capa.status,
                           client_id,
                           order.capa.valor,
                           order.capa.valor_desconto,
                           order.capa.valor_frete,
                           order.capa.valor_adicional_forma_pagt,
                           str(datetime.strptime(order.registro_status.aprovado_em.replace('T', ' '),
                                                 '%Y-%m-%d %H:%M:%S') if order.registro_status.aprovado_em is not None else ''),
                           order.capa.payment_type,
                           order.capa.flag,
                           order.capa.parcels,
                           order.capa.erp_payment_condition,
                           order.capa.tracking_code,
                           str(datetime.strptime(order.capa.delivery_forecast.replace('T', ' '),
                                                 '%Y-%m-%d %H:%M:%S') if order.capa.delivery_forecast is not None else ''),
                           order.capa.carrier,
                           order.capa.shipping_mode,
                           order.capa.channel_id,
                           job_config.get('db_seller'),
                           order.capa.erp_payment_option
                       ]))
        step = 'P.5 - Populou os dados do Pedido'
        if cursor.rowcount > 0:
            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                     f'Pedido {order.order_id} inserido', 'info', 'INTERNALIZA_PEDIDOS_JOB')
            step = 'P.6 - Antes de Executar a query'
            cursor.execute(queries.get_query_order(db_config.db_type),
                           queries.get_command_parameter(db_config.db_type, [order.order_id]))
            step = 'P.7 - Executou a Query'
            (order_id,) = cursor.fetchone()
            if order_id is None or order_id <= 0:
                cursor.close()
                raise Exception('Nao foi possivel obter o pedido inserido no banco de dados')
        else:
            cursor.close()
            raise Exception('O cliente nao foi inserido')

        # insere itens
        step = 'insere itens'
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                 f'\tPedido {order.order_id} com id semaforo {order_id}: Inserindo itens do pedido', 'info'
                 , 'INTERNALIZA_PEDIDOS_JOB')
        for item in order.items:
            cursor.execute(queries.get_insert_order_items_command(db_config.db_type),
                           queries.get_command_parameter(db_config.db_type, [
                               order_id,
                               item.sku,
                               item.erp_code,
                               item.quantity,
                               item.ean,
                               item.value,
                               item.discount,
                               item.freight_value
                           ]))

        cursor.close()
        conn.commit()
        conn.close()
        return True
    except Exception as e:
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), True,
                 f'Passo {step} - Erro durante a inserção dos dados do pedido {order.order_id}: {str(e)}', 'error'
                 , 'INTERNALIZA_PEDIDOS_JOB', str(order.order_id))
        conn.rollback()
        conn.close()
        return False


def call_order_procedures(job_config: dict, db_config: DatabaseConfig, order_id: int) -> (bool, str, str):
    client_erp_id = ''
    order_erp_id = ''
    success = True
    db = database.Connection(db_config)
    conn = db.get_conect()
    try:
        cursor = conn.cursor()
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                 f'\tPedido [{order_id}]: Executando procedure de cliente', 'info', 'INTERNALIZA_PEDIDOS_JOB')
        if db_config.is_sql_server():
            cursor.execute('exec openk_semaforo.sp_processa_cliente @pedido = ?', order_id)
            (client_erp_id,) = cursor.fetchone()
        elif db_config.is_oracle():
            client_out_value = cursor.var(str)
            cursor.callproc('OPENK_SEMAFORO.SP_PROCESSA_CLIENTE', [order_id, client_out_value])
            client_erp_id = client_out_value.getvalue()
        elif db_config.is_mysql():
            args = (order_id, 0)
            resultargs = cursor.callproc('openk_semaforo.SP_PROCESSA_CLIENTE', args)
            client_erp_id = resultargs[1]

        if client_erp_id is not None:
            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                     f'\tPedido [{order_id}]: Cliente ERP criado com sucesso: [{client_erp_id}]', 'info'
                     , 'INTERNALIZA_PEDIDOS_JOB')

            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                     f'\tPedido [{order_id}]: Executando procedure de pedido', 'info'
                     , 'INTERNALIZA_PEDIDOS_JOB')
            if db_config.is_sql_server():
                cursor.execute('exec openk_semaforo.sp_processa_pedido @pedido = ?', order_id)
                (order_erp_id,) = cursor.fetchone()
            elif db_config.is_oracle():
                order_out_value = cursor.var(str)
                cursor.callproc('OPENK_SEMAFORO.SP_PROCESSA_PEDIDO', [order_id, int(client_erp_id), order_out_value])
                order_erp_id = order_out_value.getvalue()
            elif db_config.is_mysql():
                args = (order_id, int(client_erp_id), 0)
                resultargs = cursor.callproc('openk_semaforo.SP_PROCESSA_PEDIDO', args)
                order_erp_id = resultargs[2]

            if order_erp_id is not None:
                send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                         f'\tPedido [{order_id}]: Pedido ERP criado com SUCESSO, ERP_ID:[{order_erp_id}]',
                         'info', 'INTERNALIZA_PEDIDOS_JOB')
            else:
                success = False
                cursor.execute(queries.get_log_integracao(db_config.db_type),
                               queries.get_command_parameter(db_config.db_type, [order_id]))
                log_integracao = cursor.fetchone()[0]
                send_log(job_config.get('job_name'), job_config.get('enviar_logs'), True,
                         f'\tPedido [{order_id}]: Nao foi possivel obter o id do pedido do ERP (retorno da procedure) '
                         f'- Erro {log_integracao}', 'warning', 'INTERNALIZA_PEDIDOS_JOB', str(order_id))
                raise Exception(f'Pedido [{order_id}]: Nao foi possivel obter o id do pedido do ERP (retorno da '
                                f'procedure) - Erro {log_integracao}')
        else:

            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), True,
                     f'\tPedido [{order_id}]:Nao foi possivel obter o id do cliente do ERP (retorno da procedure)',
                     'warning', 'INTERNALIZA_PEDIDOS_JOB', str(order_id))

        cursor.close()
        if success:
            conn.commit()
        else:
            conn.rollback()
        conn.close()
        return success, client_erp_id, order_erp_id
    except Exception as e:
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), True,
                 f'\tPedido [{order_id}]:Erro no método de chamada da procedure de internalização do pedido, '
                 f'ERRO: {str(e)}', 'error', 'INTERNALIZA_PEDIDOS_JOB', str(order_id))
        conn.rollback()
        conn.close()


def check_non_integrated_order_existence(db_config: DatabaseConfig, order_id: int) -> bool:
    db = database.Connection(db_config)
    conn = db.get_conect()
    try:
        cursor = conn.cursor()
        cursor.execute(queries.get_query_non_integrated_order(db_config.db_type),
                       queries.get_command_parameter(db_config.db_type, [order_id]))
        existent_order = cursor.fetchone()[0]
        cursor.close()
        conn.close()
        return existent_order is not None and existent_order > 0
    except Exception as e:
        conn.close()
        return False


# region - MPLACE
def process_order_queue_mplace(job_config: dict, status: str, db_config: DatabaseConfig,
                               status_to_insert: bool = False) -> None:
    try:
        print()
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                 f'== [INICIO] Consultando fila de pedidos no status {status}', 'info', 'INTERNALIZA_PEDIDOS_JOB')
        queue = api_Mplace.get_order_queue_mplace(src.client_data, status)

        qty = 0
        for q_order in queue:
            try:
                sleep(0.5)
                qty = qty + 1
                print()
                send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                         f'Iniciando processamento ({qty} de {len(queue)}) pedido {q_order.pedido_oking_id}', 'info',
                         'INTERNALIZA_PEDIDOS_JOB')
                order = api_Mplace.get_order_mplace(src.client_data, q_order.pedido_oking_id)

                if order.partner_order_code is not None and order.partner_order_code != '':  # Pedido integrado anteriormente

                    if check_order_existence(db_config, order.order_id):
                        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                                 f'\tPedido [{order.order_id}]: Pedido ja integrado com o ERP, chamando '
                                 f'procedure de atualizacao...', 'info', 'INTERNALIZA_PEDIDOS_JOB')
                        if call_update_order_procedure(job_config, db_config, order.order_id, order.status_code):
                            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                                     f'\tPedido [{order.order_id}]: Pedido atualizado com sucesso', 'info'
                                     , 'INTERNALIZA_PEDIDOS_JOB')
                            set_codigo_erp_order(job_config, db_config=db_config, order=order, queue_order=q_order,
                                                 order_erp_id=order.partner_order_code, client_erp_id='')
                    else:
                        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                                 f'\tPedido [{order.order_id}]: NÃO existe no banco semaforo porem ja foi integrado '
                                 f'previamente. Protocolando pedido...', 'warning', 'INTERNALIZA_PEDIDOS_JOB')
                        protocol_non_existent_order(job_config, q_order)

                else:  # Pedido nao integrado anteriormente

                    if check_order_existence(db_config, order.order_id):  # Pedido existente no banco semaforo
                        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                                 f'\tPedido [{order.order_id}]: Pedido já existente no banco semáforo, '
                                 f'porem nao integrado com erp. Chamando procedures', 'info', 'INTERNALIZA_PEDIDOS_JOB')
                        sp_success, client_erp_id, order_erp_id = call_order_procedures(job_config, db_config,
                                                                                        q_order.pedido_oking_id)
                        if sp_success:
                            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                                     f'\tPedido [{order.order_id}]: Chamadas das procedures executadas com sucesso,'
                                     f' protocolando pedido...', 'info', 'INTERNALIZA_PEDIDOS_JOB')
                            set_codigo_erp_order(job_config, db_config=db_config, order=order, queue_order=q_order,
                                                 order_erp_id=order_erp_id, client_erp_id=client_erp_id)

                    elif status_to_insert:  # Pedido nao existe no semaforo e esta em status de internalizacao (pending e paid)

                        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                                 f'Inserindo novo pedido no banco semaforo', 'info', 'INTERNALIZA_PEDIDOS_JOB')
                        inserted = insert_temp_order_mplace(job_config, order, db_config)
                        if inserted:
                            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                                     f'\tPedido [{order.order_id}]: Pedido inserido com sucesso, chamando procedures...',
                                     'info', 'INTERNALIZA_PEDIDOS_JOB')
                            sp_success, client_erp_id, order_erp_id = call_order_procedures(job_config, db_config,
                                                                                            q_order.pedido_oking_id)
                            if order.status_code != "AGUARDANDO_PAGAMENTO":
                                set_status_processing(job_config, q_order)
                            if sp_success:
                                send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                                         f'\tPedido [{order.order_id}]: Chamadas das procedures executadas com sucesso,'
                                         f' protocolando pedido...', 'info', 'INTERNALIZA_PEDIDOS_JOB')
                                set_codigo_erp_order(job_config, db_config=db_config, order=order, queue_order=q_order,
                                                     order_erp_id=order_erp_id, client_erp_id=client_erp_id)

                    else:  # Pedido nao existe no semaforo e nao esta em status de internalizacao (pending e paid)
                        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                                 f'\tPedido [{order.order_id}]: Pedido nao existente no banco semaforo e nao se '
                                 f'encontra em status de internalizacao', 'warning', 'INTERNALIZA_PEDIDOS_JOB')
                        api_Mplace.put_protocol_orders_mplace(q_order.protocolo)

            except Exception as e:
                send_log(job_config.get('job_name'), job_config.get('enviar_logs'), True,
                         f'\tPedido [{q_order.pedido_oking_id}]: Erro no processamento do pedido: {str(e)}',
                         'error', 'PEDIDO', q_order.pedido_oking_id)
                raise e
    except Exception as e:
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), True,
                 f'Erro ao inicializar job de processamento de pedidos do status {status}: {str(e)}'
                 , 'error', 'INTERNALIZA_PEDIDOS_JOB', status)
        raise e


def insert_temp_order_mplace(job_config: dict, order: OrderMplace, db_config: DatabaseConfig) -> bool:
    step = ''
    db = database.Connection(db_config)
    conn = db.get_conect()
    try:
        step = 'conexao'
        cursor = conn.cursor()

        existent_client = None

        cursor.execute(queries.get_query_cliente_cpfcnpj(db_config.db_type),
                       queries.get_command_parameter(db_config.db_type, [
                           order.customer.document_number.translate(str.maketrans('', '', string.punctuation))]))
        existent_client = cursor.fetchone()

        if existent_client is None:
            # insere cliente
            step = 'P.1 -Inserindo Novo Cliente'
            logger.info(f'')
            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                     f'\tPedido [{order.order_id}]: Inserindo cliente', 'info', 'PEDIDO')
            cursor.execute(queries.get_insert_client_command(db_config.db_type),
                           queries.get_command_parameter(db_config.db_type, [
                               order.customer.name,  # nome
                               order.customer.name,  # razao_social
                               order.customer.type,  # tipo_pessoa
                               order.customer.document_number,  # cpf_cnpj
                               order.customer.gender,  # sexo
                               order.customer.email,  # email
                               order.customer.identity_number,  # rg
                               '',  # orgao
                               str(datetime.strptime(order.customer.born_at.replace('T', ' '),
                                                     # data_nascimento_constituicao
                                                     '%Y-%m-%d %H:%M:%S') if order.customer.born_at is not None else ''),
                               order.customer.phones.office,  # telefone_residencial
                               order.customer.phones.mobile,  # telefone_celular
                               order.customer.state_registration,  # inscricao_estadual
                               '',  # inscricao_municipal
                               '',  # codigo_referencia_erp
                               '',  # codigo_representante
                               order.customer.billing.postal_code,  # end_principal_cep
                               '',  # END_PRINCIPAL_TIPOLOGRADOURO
                               order.customer.billing.address,  # end_principal_logradouro
                               order.customer.billing.number,  # end_principal_numero
                               order.customer.billing.complement,  # end_principal_complemento
                               order.customer.billing.neighborhood or " ",  # end_principal_bairro
                               order.customer.billing.city,  # end_principal_cidade
                               order.customer.billing.state,  # end_principal_estado
                               order.customer.billing.reference,  # end_principal_referencia_ent
                               order.customer.billing.city_ibge_code,  # end_principal_codigo_ibge
                               'IN'  # direcao
                           ]))
            step = 'P.2 -Populou os dados do Cliente'
            if cursor.rowcount > 0:
                send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                         f'\tPedido [{order.order_id}]: Cliente inserido/atualizado', 'info', 'INTERNALIZA_PEDIDOS_JOB')
                step = 'P.3 -Consultando o CLiente pelo CPF/CNPJ'
                cursor.execute(queries.get_query_cliente_cpfcnpj(db_config.db_type),
                               queries.get_command_parameter(db_config.db_type, [
                                   order.customer.document_number.translate(
                                       str.maketrans('', '', string.punctuation))]))
                step = 'P.4 -Query executada'
                (client_id,) = cursor.fetchone()

                send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                         f'\tPedido [{order.order_id}]: Cliente Encontrado:{client_id}', 'info'
                         , 'INTERNALIZA_PEDIDOS_JOB')

                if client_id is None or client_id <= 0:
                    cursor.close()
                    raise Exception('Nao foi possivel obter o cliente inserido do banco de dados')
            else:
                cursor.close()
                raise Exception('O cliente nao foi inserido')
        else:
            # update no cliente existente
            step = 'P.5 -Atualizando Cliente existente'
            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                     f'\tPedido [{order.order_id}]: Atualizando cliente existente', 'info', 'INTERNALIZA_PEDIDOS_JOB')
            step = 'P.6 -Antes de excutar a Query'
            cursor.execute(queries.get_update_client_sql(db_config.db_type),
                           queries.get_command_parameter(db_config.db_type, [
                               order.customer.name,  # nome
                               order.customer.name,  # razao_social
                               order.customer.type,  # tipo_pessoa
                               order.customer.document_number,  # cpf_cnpj
                               order.customer.gender,  # sexo
                               order.customer.email,  # email
                               order.customer.identity_number,  # rg
                               '',  # orgao
                               str(datetime.strptime(order.customer.born_at.replace('T', ' '),
                                                     # data_nascimento_constituicao
                                                     '%Y-%m-%d %H:%M:%S') if order.customer.born_at is not None else ''),
                               order.customer.phones.office,  # telefone_residencial
                               order.customer.phones.mobile,  # telefone_celular
                               order.customer.state_registration,  # inscricao_estadual
                               '',  # inscricao_municipal
                               '',  # codigo_referencia_erp
                               '',  # codigo_representante
                               order.customer.billing.postal_code,  # end_principal_cep
                               '',  # END_PRINCIPAL_TIPOLOGRADOURO
                               order.customer.billing.address,  # end_principal_logradouro
                               order.customer.billing.number,  # end_principal_numero
                               order.customer.billing.complement,  # end_principal_complemento
                               order.customer.billing.neighborhood or " ",  # end_principal_bairro
                               order.customer.billing.city,  # end_principal_cidade
                               order.customer.billing.state,  # end_principal_estado
                               order.customer.billing.reference,  # end_principal_referencia_ent
                               order.customer.billing.city_ibge_code or " ",  # end_principal_codigo_ibge
                               order.customer.document_number  # cpf_cnpj
                           ]))
            step = 'P.6 -Query executada'
            client_id = existent_client[0]

        # insere pedido
        step = 'P.7 -Antes de Executar a Query de Pedido'
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                 f'\tPedido [{order.order_id}]: Inserindo cabecalho pedido', 'info', 'INTERNALIZA_PEDIDOS_JOB')

        cursor.execute(queries.get_insert_b2b_order_command(db_config.db_type),
                       queries.get_command_parameter(db_config.db_type, [
                           order.order_id,  # pedido_oking_id
                           order.order_id,  # pedido_venda_id
                           order.site_order_code,  # pedido_canal
                           '',  # numero_pedido_externo
                           str(datetime.strptime(order.order_date.replace('T', ' '),  # data_pedido
                                                 '%Y-%m-%d %H:%M:%S') if order.order_date is not None else ''),
                           order.status_code,  # status
                           client_id,  # client_id
                           order.total_amount,  # valor
                           order.discount_value,  # valor_desconto
                           order.freight_value,  # valor_frete
                           0,  # valor_adicional_forma_pagt
                           str(datetime.strptime(order.status_record.approved_at.replace('T', ' '),  # data_pagamento
                                                 '%Y-%m-%d %H:%M:%S') if order.status_record.approved_at is not None else ''),
                           order.payment.payment_options.replace("'", ''),  # tipo_pagamento
                           order.payment.flag,  # bandeira
                           order.payment.plot_amount,  # parcelas
                           '',  # condicao_pagamento_erp
                           '',  # opcao_pagamento_erp
                           order.tracking_code or '',  # codigo_rastreio
                           str(datetime.strptime(order.delivery_forecast.replace('T', ' '),  # data_previsao_entrega
                                                 '%Y-%m-%d %H:%M:%S') if order.delivery_forecast is not None else ''),
                           order.tracking.carrier,  # transportadora
                           '',  # modo_envio
                           0,  # canal_id
                           0,  # loja_id
                           '',  # codigo_representante
                           order.document_intermediator or '',  # cnpj_intermediador
                           order.delivery_address.postal_code,  # end_entrega_cep
                           '',  # end_entrega_tipo_logradouro
                           order.delivery_address.address,  # end_entrega_logradouro
                           order.delivery_address.number,  # end_entrega_numero
                           order.delivery_address.complement,  # end_entrega_complemento
                           order.delivery_address.neighborhood,  # end_entrega_bairro
                           order.delivery_address.city,  # end_entrega_cidade
                           order.delivery_address.state,  # end_entrega_estado
                           order.delivery_address.reference,  # END_ENTREGA_REFERENCIA_ENT
                           order.delivery_address.city_ibge_code or ''  # end_entrega_codigo_ibge
                           #  job_config.get('db_seller')
                       ]))

        if cursor.rowcount > 0:
            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                     f'\tPedido [{order.order_id}]: Inserido com sucesso', 'info', 'INTERNALIZA_PEDIDOS_JOB')
            cursor.execute(queries.get_query_order(db_config.db_type),
                           queries.get_command_parameter(db_config.db_type, [order.order_id]))
            (order_id,) = cursor.fetchone()
            if order_id is None or order_id <= 0:
                cursor.close()
                raise Exception('Nao foi possivel obter o pedido inserido no banco de dados')
        else:
            cursor.close()
            raise Exception('O cliente nao foi inserido')

        # insere itens
        step = 'P.8 -Itens Pedido Antes Executar a Query'
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                 f'\tPedido [{order.order_id}]: Inserindo ITENS do pedido', 'info', 'INTERNALIZA_PEDIDOS_JOB')
        for item in order.itempedido:
            cursor.execute(queries.get_insert_b2b_order_items_command(db_config.db_type),
                           queries.get_command_parameter(db_config.db_type, [
                               order_id,  # pedido_id,
                               item.sku_seller_id,  # sku
                               item.sku_partner_id,  # codigo_erp
                               item.quantity,  # quantidade
                               '',  # ean
                               item.sale_price,  # valor
                               item.discount,  # valor_desconto
                               item.freight_value,  # valor_frete
                               # '',  # codigo_filial_erp
                               '',  # codigo_filial_expedicao_erp
                               '',  # CODIGO_FILIAL_FATURAMENT_ERP
                               '',  # cnpj_filial_venda
                           ]))

        cursor.close()
        conn.commit()
        conn.close()
        return True
    except Exception as e:
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), True,
                 f'Passo {step} - Erro durante a inserção dos dados do pedido {order.order_id}: {str(e)}'
                 , 'error', 'INTERNALIZA_PEDIDOS_JOB', str(order.order_id))
        conn.rollback()
        conn.close()
        return False


# endregion


def query_invoices(job_config: dict, db_config: DatabaseConfig, pedidoId: str = '') -> List[Invoice]:
    """
    Consulta as notas fiscais a serem enviadas na api do okvendas
    Args:
        job_config: Configuração do job
        db_config: Configuracao do banco de dados
        pedidoId: Id do pedido da nota fiscal

    Returns:
        Lista de notas fiscais para enviar
    """
    db = database.Connection(db_config)
    conn = db.get_conect()
    cursor = conn.cursor()
    invoices: List[Invoice] = []
    try:
        if db_config.is_oracle():
            conn.outputtypehandler = database.output_type_handler

        newsql = db_config.sql.lower()

        if src.client_data['operacao'].lower().__contains__('mplace') or\
           src.client_data['operacao'].lower().__contains__('okvendas'):
            newsql = newsql.replace("@pedido_id", f'{pedidoId}').replace('#v', ',')

        cursor.execute(newsql.replace(';', ''))
        rows = cursor.fetchall()
        columns = [col[0].lower() for col in cursor.description]
        results = [dict(zip(columns, row)) for row in rows]
        cursor.close()
        conn.close()
        if len(results) > 0:
            invoices = [Invoice(**p) for p in results]

    except Exception as ex:
        logger.error(f' ')
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), True,
                 f'Erro ao consultar notas fiscais no banco: {str(ex)}', 'error'
                 , 'envia_notafiscal_job', 'NOTA FISCAL')
        raise ex

    return invoices


def set_status_processing(job_config: dict, queue_order: Queue) -> None:
    try:
        protocoled_order = api_Mplace.post_status_processing_mplace(queue_order.pedido_oking_id)
        if protocoled_order:
            logger.info(current_job + f' | Pedido {queue_order.pedido_oking_id} protocolado via api OkVendas')
        else:
            logger.warning(
                current_job + f' | Falha ao protocolar pedido {queue_order.pedido_oking_id} via api OkVendas')
    except Exception as ex:
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), True,
                 f'Erro ao protocolar pedido {queue_order.pedido_oking_id}: {str(ex)}', 'error'
                 , 'INTERNALIZA_PEDIDOS_JOB', queue_order.pedido_oking_id)
