import logging
import src.database.connection as database
import src.database.utils as utils
from src.api.entities.produto_mplace import Produto_Mplace
from src.api.entities.produto_okvendas import Produto_Okvendas
from src.api.entities.produto_parceiro_mplace import Produto_Parceiro_Mplace
from src.database.queries import IntegrationType
from src.database.utils import DatabaseConfig
import src.api.okinghub as api_okHUB
from src.database import queries
import src
from typing import List
from src.jobs.system_jobs import OnlineLogger
from src.api import api_mplace as api_Mplace
from src.api import okvendas as api_OkVendas
from src.api.entities.foto_sku import Foto_Sku
logger = logging.getLogger()
send_log = OnlineLogger.send_log
from threading import Lock
lock = Lock()


def job_send_products(job_config_dict: dict):
    """
    Job para realizar a atualização de produtos
    Args:
        job_config_dict: Configuração do job
    """
    with lock:
        db_config = utils.get_database_config(job_config_dict)
        # Exibe mensagem monstrando que a Thread foi Iniciada
        logger.info(f'==== THREAD INICIADA -job: ' + job_config_dict.get('job_name'))

        # LOG de Inicialização do Método - Para acompanhamento de execução
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True,
                 f'Produto - Iniciado', 'exec', 'envia_produto_job', 'PRODUTO')

        if db_config.sql is None:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False,
                     f'Comando sql para criar produtos nao encontrado', 'warning', 'envia_produto_job')
            return

        if src.client_data['operacao'].lower().__contains__('mplace'):
            # products = query_products_erp_mplace(job_config_dict, db_config)
            products = query_partner_products_erp_mplace(job_config_dict, db_config)
        elif src.client_data['operacao'].lower().__contains__('okvendas'):
            produtos_repetidos = query_products(db_config)
            keys_photos_sent = {}
            photos_sku = mount_list_photos(produtos_repetidos)
            products = remove_repeat_products(produtos_repetidos)

        else:
            products = query_products_erp(job_config_dict, db_config)
        if products is not None and len(products) > 0:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True,
                     f'Total de Produto(s) a serem atualizados: {len(products)}', 'info', 'envia_produto_job', 'PRODUTO')
            if src.client_data['operacao'].lower().__contains__('okvendas'):
                for prod in products:
                    try:
                        response = api_OkVendas.post_produtos([prod])

                        for res in response:
                            if res.status > 1:
                                send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True,
                                         f'Erro ao gerar produto {res.codigo_erp} na api okvendas. '
                                         f'Erro gerado na api: {res.message}',
                                         'warning', 'envia_produto_job', prod.codigo_referencia)
                            else:
                                send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False,
                                         f'Produto {res.codigo_erp} criado com sucesso', 'info', 'envia_produto_job')
                                protocol_products(job_config_dict, products, db_config)

                                keys_photos_sent[(prod.codigo_erp, prod.preco_estoque[0].codigo_erp_atributo)] = ""

                    except Exception as e:
                        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False,
                                 f'Erro durante o envio de produtos: {str(e)}', 'error', 'envia_produto_job')

                if send_photos_products(photos_sku, keys_photos_sent, 50):
                    logger.info(job_config_dict.get('job_name') + f' | Fotos dos produtos cadastradas com sucesso')
                else:
                    print(" Erro ao cadastrar as fotos do produto")
                    logger.error(job_config_dict.get('job_name') + f' | Erro durante o cadastro das fotos dos produtos')
            else:
                api_products = []
                for product in products:

                    if api_products.__len__() < 10:
                        api_products.append(product)
                    else:
                        send_log(job_config_dict.get('job_name'), False, False
                                 , f'Enviando Pacote: {api_products.__len__()}', 'info', 'envia_produto_job'
                                 , 'PRODUTO')

                        if src.client_data['operacao'].lower().__contains__('mplace'):
                            # ***********************************
                            # response = api_Mplace.post_products_mplace(src.client_data['url_api_principal']+'/api/Product',
                            #                                            api_products, src.client_data['token_api_integracao'])
                            response = api_Mplace.post_products_mplace_partner(api_products, job_config_dict, db_config)
                            validate_response_products(response, db_config, job_config_dict)
                        else:
                            response = api_okHUB.post_products(api_products)
                            validate_response_products(response, db_config, job_config_dict)
                        api_products = []
                        send_log(job_config_dict.get('job_name'), False, False, f'Tratando retorno', 'info'
                                 , 'envia_produto_job', 'PRODUTO')


            # Se ficou algum sem processa
            if api_products.__len__() > 0:
                send_log(job_config_dict.get('job_name'), False, False,
                         f'Enviando Pacote: {api_products.__len__()}', 'info', 'envia_produto_job', 'PRODUTO')
                if (src.client_data['operacao'].lower().__contains__('mplace')):
                    # ***********************************
                    # response = api_Mplace.post_products_mplace((src.client_data['url_api_principal']+'/api/Product'), api_products, src.client_data['token_api_integracao'])
                    response = api_Mplace.post_products_mplace_partner(api_products, job_config_dict, db_config)
                else:
                    response = api_okHUB.post_products(api_products)
                    validate_response_products(response, db_config, job_config_dict)
                api_products = []
                send_log(job_config_dict.get('job_name'), False, False, f'Tratando retorno', 'info'
                         , 'envia_produto_job', 'PRODUTO')
        else:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False
                     , f'Nao existem produtos a serem enviados no momento', 'warning', 'envia_produto_job')


def query_products_erp(job_config_dict: dict, db_config: DatabaseConfig):
    """
    Consulta os produtos para atualizar no banco de dados
    Args:
        job_config_dict: Configuração do job
        db_config: Configuracao do banco de dados

    Returns:
        Lista de produtos para atualizar
    """
    db = database.Connection(db_config)
    conn = db.get_conect()
    cursor = conn.cursor()

    try:
        # monta query com EXISTS e NOT EXISTS
        # verificar se já possui WHERE
        clause = "where"
        if ("where" in db_config.sql.lower()):
            clause = "and"
        newsql = f"""{db_config.sql.lower().replace("from", ", 1 as existe from")} {clause} exists ({db_config.semaforo_sql})
                      union 
                     {db_config.sql.lower().replace("from", ", 0 as existe from")} {clause} not exists ({db_config.semaforo_sql.lower().replace("and s.data_alteracao"," -- and s.data_alteracao")})"""
        cursor.execute(newsql.lower().replace('#v', ','))
        rows = cursor.fetchall()
        columns = [col[0] for col in cursor.description]
        results = list(dict(zip(columns, row)) for row in rows)
        cursor.close()
        conn.close()

        products = []
        if len(results) > 0:
            products = product_dict(results)
        return products
    except Exception as ex:
        if src.exibir_interface_grafica:
            raise
        else:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False
                     ,f' Erro ao consultar produtos no banco semaforo: {str(ex)}', 'error', 'envia_produto_job')


def product_dict(products):
    lista = []
    for row in products:
        pdict = {
            'nome': str(row['nome'])
            , 'codigo_sku': str(row['codigo_sku_variacao'])
            , 'codigo_erp': str(row['codigo_erp'])
            , 'codigo_sku_principal': str(row['codigo_sku_principal'])
            , 'codigo_erp_variacao': str(row['codigo_erp_variacao'])
            , 'agrupador': str(row['agrupador'])
            , 'ean_13_variacao': str(row['ean_13_variacao'])
            , 'ean_14_variacao': str(row['ean_14_variacao'])
            , 'variacao_opcao_1': str(row['variacao_opcao_1'])
            , 'variacao_opcao_valor_1': str(row['variacao_opcao_valor_1'])
            , 'variacao_opcao_2': str(row['variacao_opcao_2'])
            , 'variacao_opcao_valor_2': str(row['variacao_opcao_valor_2'])
            , 'variacao_opcao_3': str(row['variacao_opcao_3'])
            , 'variacao_opcao_valor_3': str(row['variacao_opcao_valor_3'])
            , 'descricao': str(row['descricao'])
            , 'metakeyword': str(row['palavra_chave'])
            , 'metadescription': str(row['meta_descricao'])
            , 'quantidade_minima': int(row['quantidade_minima'])
            , 'quantidade_caixa': int(row['quantidade_caixa'])
            , 'multiplo': bool(row['multiplo'])
            , 'ativo': bool(row['ativo'])
            , 'peso': round(float(row['peso']), 1)
            , 'volume': round(float(row['volume']), 1)
            , 'largura': round(float(row['largura']), 1)
            , 'altura': round(float(row['altura']), 1)
            , 'comprimento': round(float(row['comprimento']), 1)
            , 'marca_codigo': str(row['marca_codigo'])
            , 'marca_descricao': str(row['marca_descricao'])
            , 'fabricante_codigo': str(row['fabricante_codigo'])
            # , 'fabricante_descricao': str(row['fabricante_descricao'])
            , 'fabricante_link_imagem': str(row['fabricante_link_imagem'])
            , 'medida_descricao': str(row['medida_descricao'])
            , 'medida_codigo': str(row['medida_codigo'])
            , 'medida_sigla': str(row['medida_sigla'])
            , 'codigo_categoria_n1': str(row['codigo_categoria_n1'])
            , 'nome_categoria_n1': str(row['nome_categoria_n1'])
            , 'codigo_categoria_n2': str(row['codigo_categoria_n2'])
            , 'nome_categoria_n2': str(row['nome_categoria_n2'])
            , 'codigo_categoria_n3': str(row['codigo_categoria_n3'])
            , 'nome_categoria_n3': str(row['nome_categoria_n3'])
            , 'codigo_categoria_n4': str(row['codigo_categoria_n4'])
            , 'nome_categoria_n4': str(row['nome_categoria_n4'])
            , 'modelo': str(row['modelo'])
            , 'meses_garantia': int(row['meses_garantia'])
            , 'ncm': str(row['ncm'])
            , 'ipi': int(row['ipi'])
            , 'estoque_minimo': int(row['estoque_minimo'])
            , 'data_desativacao': None
            , 'data_alteracao': None
            , 'data_sincronizacao': None
            , 'tempo_adicional_entrega': int(row['tempo_adicional_entrega'])
            , 'adicional_descricao_1': str(row['adicional_descricao_1'])
            , 'adicional_conteudo_1': str(row['adicional_conteudo_1'])
            #  , 'preco_custo': int(row['preco_custo'])
            , 'token': str(src.client_data.get('token_oking'))
        }
        lista.append(pdict)

    return lista


def validate_response_single_product(identificador: str, identificador2: str, mensagem: str, db_config, job_config_dict):
    send_log(job_config_dict.get('job_name'), False, False, f'SINGLE- Inserir na Semaforo'
             , 'info', 'envia_produto_job', '')

    print(f'========================================================================')
    print(f' identificador:{identificador},   Identificado2:{identificador2}, TIPO:{IntegrationType.PRODUTO.value}'
          f', MSG:{mensagem}')

    conexao = database.Connection(db_config)
    conn = conexao.get_conect()
    cursor = conn.cursor()

    try:
        qry = queries.get_insert_update_semaphore_command(db_config.db_type)
        # print(qry)
        cursor.execute(qry,
                       queries.get_command_parameter(db_config.db_type, [identificador, identificador2, mensagem, 1]))
    except Exception as e:
        send_log(job_config_dict.get('job_name')
                 , job_config_dict.get('enviar_logs')
                 , True
                 , f'Erro ao atualizar Produto do sku: {identificador}, Erro: {str(e)}'
                 , 'error'
                 , 'PRODUTO'
                 , f'{identificador}-{identificador2}')

    cursor.close()
    conn.commit()
    conn.close()


def query_products_erp_mplace(job_config: dict, db_config: DatabaseConfig):
    db = database.Connection(db_config)
    conn = db.get_conect()
    cursor = conn.cursor()
    product: List[Produto_Mplace] = []
    try:
        if db_config.is_oracle():
            conn.outputtypehandler = database.output_type_handler
        newsql = db_config.sql.lower()
        cursor.execute(newsql.lower().replace(';', '').replace('#v', ','))
        rows = cursor.fetchall()
        columns = [col[0].lower() for col in cursor.description]
        results = [dict(zip(columns, row)) for row in rows]
        cursor.close()
        conn.close()
        if len(results) > 0:
            product = [Produto_Mplace(**p) for p in results]
    except Exception as ex:
        logger.error(f' ')
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                 f'Erro ao consultar produtos no banco: {str(ex)}', 'error', 'envia_produto_job')

    return product


def query_partner_products_erp_mplace(job_config: dict, db_config: DatabaseConfig):
    db = database.Connection(db_config)
    conn = db.get_conect()
    cursor = conn.cursor()
    partner_product: List[Produto_Parceiro_Mplace] = []
    try:
        if db_config.is_oracle():
            conn.outputtypehandler = database.output_type_handler
        newsql = utils.final_query(db_config)
        conexao = database.Connection(db_config)
        conn = conexao.get_conect()

        cursor.execute(newsql.lower().replace(';', ''))
        rows = cursor.fetchall()
        columns = [col[0].lower() for col in cursor.description]
        results = [dict(zip(columns, row)) for row in rows]
        cursor.close()
        conn.close()
        if len(results) > 0:
            partner_product = [Produto_Parceiro_Mplace(**p) for p in results]
    except Exception as ex:
        logger.error(f' ')
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                 f'Erro ao consultar produtos no banco: {str(ex)}', 'error', 'envia_produto_job')
        raise ex

    return partner_product

# def insert_produto(db_config: DatabaseConfig, product_code: str):
#     db = database.Connection(db_config)
#     conn = db.get_conect()
#     cursor = conn.cursor()
#     try:
#         cursor.execute(queries.get_insert_produto_command(db_config.db_type),
#                        queries.get_command_parameter(db_config.db_type, [
#                        codigo_erp,
#                        codigo_erp_sku,
#                        data_atualizacao,
#                        data_sincronizacao,
#                        data_envio_foto]
#                         ))
#         cursor.close()
#         conn.commit()
#         conn.close()
#     except Exception as ex:
#         print(f'Erro {ex} ao atualizar a tabela do pedido {product_code}')
#         raise ex

def validate_response_products(response, db_config, job_config_dict):
    if response is not None:
        conexao = database.Connection(db_config)
        conn = conexao.get_conect()
        cursor = conn.cursor()

        # Percorre todos os registros
        for item in response:
            if src.client_data['operacao'].lower().__contains__('mplace'):
                identificador = item.product_code
                identificador2 = item.product_sku
            else:
                identificador = item.identificador
                identificador2 = item.identificador2

            print( f'{item.identificador}-{item.identificador2}')
            if item.sucess == 1 or item.sucess == 'true':
                try:
                    cursor.execute(queries.get_insert_update_semaphore_command(db_config.db_type),
                                   queries.get_command_parameter(db_config.db_type, [identificador, identificador2,
                                                                                     IntegrationType.PRODUTO.value]))
                    # atualizados.append(response['identificador'])
                except Exception as e:
                    send_log(job_config_dict.get('job_name')
                             , job_config_dict.get('enviar_logs')
                             , True
                             , f'Erro inserir Produto: {item.identificador}, Erro: {str(e)}'
                             , 'error'
                             , 'FOTO'
                             , f'{item.identificador}-{item.identificador2}')
            else:
                send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True,
                         f'Erro ao inserir Produto: {identificador}, {item.mensagem}', 'warning',
                         'FOTO', identificador)

        cursor.close()
        conn.commit()
        conn.close()

def query_products(db_config: DatabaseConfig):
    """
    Consulta os produtos contidos no banco semaforo juntamente com os dados do banco do ERP
    Args:
        db_config: Configuracao do banco de dados

    Returns:
        Lista de produtos
    """
    # abre a connection com o banco
    db = database.Connection(db_config)
    connection = db.get_conect()
    # connection.start_transaction()
    cursor = connection.cursor()

    # obtem os dados do banco
    # logger.warning(query)
    cursor.execute(db_config.sql.replace('#v', ','))
    columns = [col[0] for col in cursor.description]
    results = [dict(zip(columns, row)) for row in cursor.fetchall()]
    cursor.close()
    connection.close()

    produtos = []
    for result in results:
        produtos.append(Produto_Okvendas(result))

    return produtos

def mount_list_photos(products: List[Produto_Okvendas]):
    photos = {}

    for product in products:
        # Grouping by codigo_erp and codigo_erp_variacao
        codigo_erp = product.codigo_erp
        codigo_erp_variacao = product.preco_estoque[0].codigo_erp_atributo

        key = (codigo_erp, codigo_erp_variacao)

        if key in photos:

            length_photos = len(photos[key])

            if contains_photo(photos[key], product.imagem_base64):
                continue
            else:
                order_photo = length_photos + 1
                photo_sku = Foto_Sku(product.imagem_base64, codigo_erp, f'{codigo_erp}_{order_photo}', order_photo,
                                      False)
                photos[key].append(photo_sku)

        else:
            # Mount photo
            photo_sku = Foto_Sku(product.imagem_base64, codigo_erp, f'{codigo_erp}_{1}', 1,
                                  True)

            photos[key] = [photo_sku]

    return photos

def remove_repeat_products(products: List[Produto_Okvendas]):
    product_keys = {}
    result_products = []

    for product in products:
        # Grouping by codigo_erp and codigo_erp_variacao
        codigo_erp = product.codigo_erp
        codigo_erp_variacao = product.preco_estoque[0].codigo_erp_atributo

        key = (codigo_erp, codigo_erp_variacao)

        if key in product_keys:
            continue
        else:
            product_keys[key] = ""
            result_products.append(product)

    return result_products

def contains_photo(photos: List[Foto_Sku], imagem_base64: str):
    for photo in photos:
        if photo.base64_foto == imagem_base64:
            return True
    return False


def protocol_products(job_config_dict: dict, products: list, db_config: DatabaseConfig) -> None:
    """
    Protocola no banco semaforo os produtos que foram enviados para a api okvendas
    Args:
        job_config_dict: Configuração do job
        products: Lista de produtos enviados para a api okvendas
        db_config: Configuracao do banco de dados
    """
    try:
        if len(products) > 0:
            db = database.Connection(db_config)
            connection = db.get_conect()
            cursor = connection.cursor()
            for prod in products:
                try:
                    dados_produto = [prod.codigo_erp, prod.preco_estoque[0].codigo_erp_atributo]
                    send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Protocolando codigo_erp {dados_produto[0]} sku {dados_produto[1]}', 'info', 'PRODUTO')
                    cursor.execute(queries.get_product_protocol_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, dados_produto))
                    send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Linhas afetadas {cursor.rowcount}', 'info', 'PRODUTO')
                except Exception as e:
                    send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Erro ao protocolar sku {prod["codigo_erp_sku"]}: {str(e)}', 'error', 'PRODUTO')
            cursor.close()
            connection.commit()
            connection.close()
    except Exception as e:
        raise e

def send_photos_products(photos_sku: dict, keys_photos_sent: dict, limit_photos_sent: int):
    count_success = 0
    list_photos = []
    for photo_key in photos_sku:

        if photo_key in keys_photos_sent:
            list_photo = photos_sku[photo_key]

            for photo in list_photo:
                if (len(list_photos) + 1) <= limit_photos_sent:
                    list_photos.append(photo)

                if len(list_photos) == limit_photos_sent:
                    success = api_OkVendas.put_photos_sku(list_photos)
                    if not success:
                        return success
                    else:
                        count_success = count_success + 1
                        list_photos = []

    if 0 < len(list_photos) <= 50:
        return api_OkVendas.put_photos_sku(list_photos)

    return count_success > 0 if True else False