import logging
import src.database.connection as database
import src.database.utils as utils
import src.api.api_mplace as api_Mplace
from src.database.queries import IntegrationType
from src.jobs.system_jobs import OnlineLogger
from src.database.utils import DatabaseConfig
import src.api.okinghub as api_okHUB
import src.api.okvendas as api_okVendas
from src.api import slack
from src.database import queries
import src
from threading import Lock
logger = logging.getLogger()
send_log = OnlineLogger.send_log
lock = Lock()


def job_send_stocks(job_config_dict: dict):
    """
    Job para realizar a atualização dos estoques padrão
    Args:
        job_config_dict: Configuração do job
    """
    with lock:
        global depositoMplace
        db_config = utils.get_database_config(job_config_dict)
        # Exibe mensagem monstrando que a Thread foi Iniciada
        logger.info(f'==== THREAD INICIADA -job: ' + job_config_dict.get('job_name'))

        stocks = query_stocks_erp(job_config_dict, db_config)

        # LOG de Inicialização do Método - Para acompanhamento de execução
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True,
                 f'Estoque - Iniciado', 'exec', 'job_send_stocks', 'ESTOQUE')

        # atualizados = []
        if stocks is not None and len(stocks) or 0 > 0:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True,
                     f'Total de estoques a serem atualizados: {len(stocks)}', 'info', 'job_send_stocks', 'ATUALIZAÇÃO')

            api_stocks = []
            for stock in stocks:
                try:
                    # time.sleep(1)
                    if api_stocks.__len__() < 50:
                        api_stocks.append(stock)
                    else:
                        send_log(job_config_dict.get('job_name'), False, False, f'Enviando Pacote: {api_stocks.__len__()}',
                                 'info', 'ESTOQUE', 'ESTOQUE')
                        if src.client_data['operacao'].lower().__contains__('okvendas'):
                            conexao = database.Connection(db_config)
                            conn = conexao.get_conect()
                            cursor = conn.cursor()
                            response = api_okVendas.send_stocks(api_stocks)
                            cursor.execute(queries.get_stock_protocol_command(db_config.db_type),
                                           queries.get_command_parameter(db_config.db_type, response['codigo_erp']))
                            cursor.close()
                            conn.commit()
                            conn.close()
                        elif not src.client_data['operacao'].lower().__contains__('mplace'):
                            response = api_okHUB.send_stocks_hub(api_stocks)
                        else:
                            response = api_Mplace.send_stocks_mplace(api_stocks)
                            depositoMplace = api_stocks[0]['dc_code'] #Procurar maneira de fazer get do deposito
                        api_stocks = []
                        send_log(job_config_dict.get('job_name'), False, False, f'Tratando retorno', 'info', 'ESTOQUE'
                                 , 'ESTOQUE')
                        validade_response_stocks(response, db_config, job_config_dict)

                except Exception as e:
                    send_log(job_config_dict.get('job_name')
                             , job_config_dict.get('enviar_logs')
                             , True
                             , f'Erro genérico ao atualizar o estoque, Erro: {str(e)}'
                             , 'error'
                             , 'ESTOQUE'
                             , 'erro_estoque')

            # Se ficou algum sem processa
            if api_stocks.__len__() > 0:
                send_log(job_config_dict.get('job_name'), False, False, f'Enviando Pacote: {api_stocks.__len__()}', 'info',
                         'ESTOQUE', 'ESTOQUE')
                if src.client_data['operacao'].lower().__contains__('okvendas'):
                    response = api_okVendas.send_stocks(api_stocks)
                elif not src.client_data['operacao'].lower().__contains__('mplace'):
                    response = api_okHUB.send_stocks_hub(api_stocks)
                else:
                    response = api_Mplace.send_stocks_mplace(api_stocks)
                    depositoMplace = api_stocks[0]['dc_code']#Procurar maneira de fazer get do deposito
                send_log(job_config_dict.get('job_name'), False, False, f'Tratando retorno', 'info', 'ESTOQUE', 'ESTOQUE')
                validade_response_stocks(response, db_config, job_config_dict)
        else:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True,
                     f'Nao ha produtos para atualizar estoque no momento', 'warning', 'ESTOQUE', 'ESTOQUE')


def query_stocks_erp(job_config_dict: dict, db_config: DatabaseConfig):
    """
    Consulta no banco de dados os estoques pendentes de atualização padrão
    Args:
        job_config_dict: Configuração do job
        db_config: Configuração do banco de dados

    Returns:
    Lista de estoques para realizar a atualização
    """
    stocks = None
    if db_config.sql is None or db_config.sql == '':
        slack.register_warn("Query estoque de produtos nao configurada!")
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True,
                 f'Query estoque de produtos nao configurada', 'warning', 'ESTOQUE', 'ESTOQUE')
    else:
        try:
            # monta query com EXISTS e NOT EXISTS
            # verificar se já possui WHERE
            newsql = utils.final_query(db_config)
            conexao = database.Connection(db_config)
            conn = conexao.get_conect()
            cursor = conn.cursor()

            print("========================================")
            print(db_config.sql)
            print("========================================")

            cursor.execute(newsql.lower().replace('#v', ','))

            print("========================================")
            print("=== RETORNOU DO BANCO ")
            rows = cursor.fetchall()
            # print(rows)
            columns = [col[0] for col in cursor.description]
            results = [dict(zip(columns, row)) for row in rows]

            cursor.close()
            conn.close()
            stocks = []
            if len(results) > 0:
                stocks = stock_dict(results)

        except Exception as ex:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True, f'{str(ex)}', 'error',
                     'ESTOQUE', 'ESTOQUE')
            if src.exibir_interface_grafica:
                raise ex
            else:
                logger.error(f'Falha na conexão com o bando de dados: {str(ex)}')

    return stocks


def stock_dict(stocks):
    lista = []
    for row in stocks:
        if src.client_data['operacao'].lower().__contains__('mplace'):
            pdict = {
                'sku_seller_id': str(row['sku']),
                'variation_option_id': int(row['variation_id']),
                'dc_code': str(row['deposito']),
                'quantity': int(row['quantidade']),
                'deactivation_date': ""
            }
        else:
            pdict = {
                'sku': str(row['sku']),
                'token': str(src.client_data.get('token_oking')),
                'quantidade': int(row['quantidade']),
                'deposito': str(row['deposito'])
            }

        lista.append(pdict)

    return lista


def validade_response_stocks(response, db_config, job_config_dict):
    if response is not None:
        conexao = database.Connection(db_config)
        conn = conexao.get_conect()
        cursor = conn.cursor()

        print(f'=== Operacao: ' + src.client_data['operacao'].lower())
        # Percorre todos os registros
        if src.client_data['operacao'].lower().__contains__('mplace'):
            for item in response:
                identificador = item.sku_seller_id
                identificador2 = depositoMplace #colocar variavel Deposito
                if item.success == 1 or item.success == 'true' or \
                        item.message == 'Estoque não atualizado, produto não encontrado ou desativado!':
                    msgret = 'SUCESSO'
                else:
                    msgret = item.message
                    send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True,
                             f'Erro ao atualizar estoque para o sku: {identificador},'
                             f'Deposito:{identificador2} {msgret}',
                             'warning',
                             'ESTOQUE', 'ESTOQUE')
                try:
                    cursor.execute(queries.get_insert_update_semaphore_command(db_config.db_type),
                                   queries.get_command_parameter(db_config.db_type, [identificador, identificador2,
                                                                                     IntegrationType.ESTOQUE.value,
                                                                                     msgret]))
                    # atualizados.append(response['identificador'])
                except Exception as e:
                    send_log(job_config_dict.get('job_name')
                             , job_config_dict.get('enviar_logs')
                             , True
                             ,
                             f'Erro ao atualizar estoque do sku: {identificador}, Deposito:{identificador2},'
                             f'Erro: {str(e)}'
                             , 'error'
                             , 'ESTOQUE'
                             , 'ESTOQUE')

        else:
            for item in response:
                identificador = item.identificador
                identificador2 = item.identificador2
                if item.sucesso == 1 or item.sucesso == 'true':
                    msgret = 'SUCESSO'
                else:
                    msgret = item.mensagem
                    send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True,
                             f'Erro ao atualizar estoque para o sku: {identificador},'
                             f'Deposito:{identificador2} {msgret}',
                             'warning',
                             'ESTOQUE', 'ESTOQUE')
                # Mesmo se der erro retira da Fila, mas grava a mensagem no Banco
                try:
                    cursor.execute(queries.get_insert_update_semaphore_command(db_config.db_type),
                                   queries.get_command_parameter(db_config.db_type, [identificador, identificador2,
                                                                                     IntegrationType.ESTOQUE.value,
                                                                                     msgret]))
                    # atualizados.append(response['identificador'])
                except Exception as e:
                    send_log(job_config_dict.get('job_name')
                             , job_config_dict.get('enviar_logs')
                             , True
                             ,
                             f'Erro ao atualizar estoque do sku: {item.identificador}, Deposito:{item.identificador2}, '
                             f'Erro: {str(e)} '
                             , 'error'
                             , 'ESTOQUE'
                             , 'ESTOQUE')


        cursor.close()
        conn.commit()
        conn.close()
